#include "PWMit.h"

#define calcPerLoop 100 //check if the output should be on every seconds/this
                        //calculate frequency and duration will need to be tested
                        // since it will depend on SSR drive and light response
                        // SSR is VO14642AT
                        // DIO pullup is 10k to 5V
                        // unsure of DIO Iol...
                        // turnon time is 550us, turnoff is 18us

PWMit :: PWMit(DigitalOutput *port, float seconds) :
    notify(PWMit::calcPWM, this),
    output(port)
{
    init(seconds);
}

PWMit :: PWMit(int port, float seconds) :
    notify(PWMit::calcPWM, this),
    output(new DigitalOutput(port))
{
    init(seconds);
}

void PWMit::init(float seconds)
{
    notify.StartPeriodic(seconds/calcPerLoop);
    secPerLoop = seconds;             
    curPercent = 0.0;
    enabled = false;
    count = 0;

    output->Set(1);  //should be 'light off'    
}

PWMit :: ~PWMit()
{
    notify.Stop(); //stop the notifier
    output->Set(1); //make sure light is off
    delete output;
}

void PWMit :: setPWM(int percent)
{
    curPercent = percent;
    return;
}

void PWMit :: disablePWM()
{
    enabled = false;
    notify.Stop();  
}

void PWMit :: enablePWM()
{
    enabled = true;
    count = 0;
    notify.StartPeriodic(secPerLoop/calcPerLoop);  
}

float PWMit :: getPWM()
{
    return curPercent;
}

void PWMit :: calcPWM(void *this_ptr)
{
    PWMit *t = (PWMit *)this_ptr;
    t->calcPWM_impl();
}
void PWMit :: calcPWM_impl()
{
    if (enabled)
    {
        if (++count >= calcPerLoop)
        {
            count = 0;
        }
        if (curPercent >= count)
            output->Set(0);
        else
            output->Set(1);      
    }
    else
    {
        output->Set(1);
        count = 0;
    }
}
