#include "Resources.h"
#include "CounterBase.h"
#include "ThunderArm.h"
#include "Math.h"
#define fabs(a) ((a)>0?(a):-(a))

ThunderArm::ThunderArm(ThunderDeployer* deploypointer)
{
	deployer = deploypointer;
	armMotor = new ThunderCANJaguar(CANID_ARM);
	armEncoderLeft = new Encoder(DIG_ARM_ENC_LEFT_A, DIG_ARM_ENC_LEFT_B, true, CounterBase::k4X);
	armEncoderRight = new Encoder(DIG_ARM_ENC_RIGHT_A, DIG_ARM_ENC_RIGHT_B, false, CounterBase::k4X);
	armCalibLimit = new AnalogChannel(ANA_ARM_PHOTO_HOME);
	armFrontLimit = new DigitalInput(DIG_ARM_PHOTO_FRONT);
	armBackLimit  = new DigitalInput(DIG_ARM_PHOTO_BACK);
	armSpeed = 0;
	preset = PRE_NONE;
	preset_angle = 0;
	adhoc_angle	= 0;
	adhocing = false;
	adhoc_disabled = false;
	actual_angle = 0;
	invert = false;
	preset_done = false;
	encoder_swapped = false;
	encoder_broken = false;
	start_match = false;
	calib_limit_broken = false;
	front_limit_broken = false;
	back_limit_broken = false;
	angle_error_Left = 0;
	angle_error_Right = 0;
	ARM_MIN_SPEED = .2;
	
	
	armEncoderLeft->Start();
	armEncoderRight->Start();

// Straight up in the air 
#define STRAIGHT_UP 665 //655
	//PRESET ANGLES (NO INVERSION)
#define LOWER_LIMIT 115 //98
	float base = LOWER_LIMIT;
	presetData [1] = base + 73;	   //LOW PEG
	presetData [2] = base + 117;   //LOW PEG OFFSET
	presetData [3] = base + 217;   //MEDIUM PEG
	presetData [4] = base + 262;   //MEDIUM PEG OFFSET
	presetData [5] = base + 385;   //HIGH PEG
	presetData [6] = base + 436;   //HIGH PEG OFFSET
	presetData [7] = base + 163;   //FEEDER STATION
#define CALIB_LIMIT_LOCATION	(LOWER_LIMIT + 50)
	presetData [8] = CALIB_LIMIT_LOCATION;     //FLOOR
	presetData [9] = base + 553;   //Straight up
	presetData [10] = base + 482;	//AUTO HIGH PEG OFFSET
	presetData [11] = base + 376;	//AUTO HIGH PEG OFFSET LOWER
	presetData [12] = base + 292;	//AUTO MED PEG OFFSET
	presetData [13] = base + 181;	//AUTO MED PEG OFFSET LOWER
	presetData [14] = base + 221;	//AUTO LOW PEG OFFSET
	presetData [15] = base + 57;	//AUTO LOW PEG OFFSET LOWER
	presetData [16] = base + 490;	//AUTO HIGH PEG
	presetData [17] = base + 315;	//AUTO HIGH PEG LOWER
	presetData [18] = base + 270;	//AUTO MED PEG
	presetData [19] = base + 152;	//AUTO MED PEG LOWER
	presetData [20] = base + 191;	//AUTO LOW PEG
	presetData [21] = base + 20;	//AUTO LOW PEG LOWER
	
	//PRESET ANGLES (INVERTED)
	base = 1205; //1195;
	presetDataInverted [1] = base - 96; 		//LOW PEG
	presetDataInverted [2] = base - 112; 		//LOW PEG OFFSET
	presetDataInverted [3] = base - 244; 		//MEDIUM PEG
	presetDataInverted [4] = base - 268; 		//MEDIUM PEG OFFSET
	presetDataInverted [5] = base - 428; 		//HIGH PEG
	presetDataInverted [6] = base - 472; 		//HIGH PEG OFFSET
	presetDataInverted [7] = base - 188;		//FEEDER STATION
	presetDataInverted [8] = base - 0;      	//FLOOR
	presetDataInverted [9] = base - 544;		//Straight up
}
//MINIMUM: 24
//MAXIMUM: 302
//DIFFERA: 278
//MIDPOIN: 163

ThunderArm::~ThunderArm()
{
	delete armMotor;
	delete armEncoderLeft;
	delete armEncoderRight;
	delete armCalibLimit;
	delete armFrontLimit;
	delete armBackLimit;
}

void ThunderArm::ArmProcess()
{
	float temp_angle = 0;
	
	if(encoder_broken)
	{
		armMotor->Set(armSpeed); 
		preset_done = true;
		start_match = false;
		return;
	}
	
	//CALIBRATION CODE
//	if(armCalibLimit->GetVoltage()>3 && !calib_limit_broken)
	if (armFrontLimit->Get() && !calib_limit_broken)
	{
		/* 
		 * This is the number our encoder reading 
		 * is off of what it should be at the limit
		 * switch. Number is determined by a reading
		 * when the sensor is working. If broken it
		 * will just go with whatever reading it is
		 * at.
		 */
		
		angle_error_Left = CALIB_LIMIT_LOCATION - armEncoderLeft->GetRaw();
		angle_error_Right = CALIB_LIMIT_LOCATION - armEncoderRight->GetRaw();
	} else if (calib_limit_broken) {
		angle_error_Left = angle_error_Right = 0;
	}
	
	//Calculate encoder angle, taking error into account
	if(!encoder_swapped)
	{
		actual_angle = armEncoderLeft->GetRaw();
		actual_angle += angle_error_Left;
	}
	else
	{
		actual_angle = armEncoderRight->GetRaw();
		actual_angle += angle_error_Right;
	}

	// START OF DC FORMULA:  Note this wasn't symmetric
	//ARM_MIN_SPEED = ARM_MIN_SPEED_BASE + ((sin(6.183*((float)(actual_angle) + ARM_START_ANGLE)/1440) + 1)/2) * ARM_MIN_SPEED_MOD;
	ARM_MIN_SPEED = 0.07 + (0.047 * fabs(sin(6.28 * ((float)(actual_angle + 45) / 1440.0))));
	//ARM_MIN_SPEED_BASE is the speed when the cos equation is equal to 0
	//The cos is taken after the encoder ticks are translated such that the arm at 90 degrees is 0 degrees in the equation
		//C++ uses radians not degrees, so after translation, the equation is multiplied by pi/720 (720 instead of 180 because it's in ticks not degrees)
	//The absolute value (fabs) is taken because direction isn't determined in min speed, and the value needs to get larger not smaller past 90 degrees
	//ARM_MIN_SPEED_MOD is the max value the cos equation can give
	//ARM_MIN_SPEED = ARM_MIN_SPEED_BASE + (fabs(sin(((float)(actual_angle) + ARM_START_ANGLE) * 3.1415296 / 180 / ARM_ENCODING)) * ARM_MIN_SPEED_MOD);
	
	//All code for the match start deployment
	if(start_match)
	{	
		//printf("In Start Match\n");
		if(actual_angle < ARM_DEPLOY)
		{
			temp_angle = (float)(ARM_DEPLOY - actual_angle)/(float)ARM_DEPLOY;
			float mv = ARM_DEPLOY_MIN_SPEED + (ARM_DEPLOY_MAX_SPEED - ARM_DEPLOY_MIN_SPEED)*
            										((1 - temp_angle)*(1 - temp_angle));
			if (mv < ARM_DEPLOY_MIN_SPEED)
				mv = ARM_DEPLOY_MIN_SPEED;
			if(actual_angle <= 1 * ARM_ENCODING)
				mv = 1.0f;
			armMotor->Set(mv);
			return;
		}
		else
		{
			if(actual_angle < ARM_DEPLOY2)
			{
				temp_angle = (float)(ARM_DEPLOY2 - actual_angle)/(float)(ARM_DEPLOY2-ARM_DEPLOY);
				float mv = ARM_DEPLOY_MIN_SPEED + (ARM_DEPLOY_MAX_SPEED - ARM_DEPLOY_MIN_SPEED) * temp_angle;
				if (mv < ARM_DEPLOY_MIN_SPEED)
					mv = ARM_DEPLOY_MIN_SPEED;
				armMotor->Set(mv);
				return;
			}
			else
			{
				if(actual_angle < ARM_DEPLOY3)
				{
					armMotor->Set(ARM_DEPLOY_MAX_SPEED);
					return;
				}
				else
				{
					armMotor->Set(0);
					if(preset == PRE_NONE)
						preset_angle = actual_angle;
					start_match = false;
				}
			}
		}
	}
	
	/*
	 * All code for overrides
	 * Ex: Minibot and tusks zones
	 *     Limit switches
	 */
	int overrideReturn = ArmCheckOverrides();
	if(overrideReturn != 0)
	{
		//printf("limits\n");
		if(preset != PRE_NONE){
			if (overrideReturn > 0) 
			{
				armMotor->Set(0.2);
				return;
			}
			if (overrideReturn < 0)
			{
				armMotor->Set(-0.2);
				return;
			}
		}
		if(armSpeed > ARM_MIN_SPEED && overrideReturn > 0)
			armMotor->Set(armSpeed); 
		if(armSpeed < -ARM_MIN_SPEED && overrideReturn < 0)
			armMotor->Set(armSpeed); 
		return;
	}
	
	
	//Code for manual movement
	if(preset == PRE_NONE && !adhocing)
	{
		//printf("Manual\n");
		if(actual_angle < ARM_CLOSE_TO_LOW)
		{
			if (armSpeed < -ARM_MIN_SPEED)
				armSpeed = -ARM_MIN_SPEED;
		}
		if(actual_angle > ARM_CLOSE_TO_HIGH)
		{
		    if (armSpeed > ARM_MIN_SPEED)
		    	armSpeed = ARM_MIN_SPEED;
		}
		
		if (actual_angle < STRAIGHT_UP && armSpeed < 0)
			armSpeed  = armSpeed * 0.5;
		else if (actual_angle > STRAIGHT_UP && armSpeed > 0)
			armSpeed = armSpeed * 0.5;

		if(armSpeed > 0 && armSpeed < ARM_MIN_STALL_SPEED)
			armSpeed = ARM_MIN_STALL_SPEED;
		if(armSpeed < 0 && armSpeed > -ARM_MIN_STALL_SPEED)
			armSpeed = -ARM_MIN_STALL_SPEED;
		armMotor->Set(armSpeed); 
		preset_done = true;
		//printf("Arm Speed: %f\n", armSpeed);
	}
	else
	{
		
		/*Pseudocode:
		 * 
		 * If actual within range of goal -> do nothing
		 * if actual greater/less than preset -> 
		 * 		move towards preset, ramp power based on distance away, with a minimum
		 */
		
		//Preset calculations
		if(preset_angle < presetData[8])
			preset_angle = presetData[8];
		if(preset_angle > presetDataInverted[8])
			preset_angle = presetDataInverted[8];
			
		
		if(ArmWithinRange(actual_angle, ARM_TOLERANCE, preset_angle))
		{
			if (actual_angle < 618)
				armMotor->Set(.12);
			else if (actual_angle > 718)
				armMotor->Set(-0.12);
			else
				armMotor->Set(0);
					
			//armMotor->Set(-1*ARM_MIN_STALL_SPEED*(((float)(actual_angle-(163 * ARM_ENCODING)))/(139 * ARM_ENCODING)));
			//armMotor->Set(sin(6.183*(((float)(actual_angle) + (float)ARM_START_ANGLE))/1440) * ARM_MIN_STALL_SPEED);
			
			preset_done = true;
			//printf("at preset %f\n", preset_angle);
		}
		else 
		{
			preset_done = false;
			//printf("not at preset: PresetNum:%i PresetAngle:%f CurAngle:%f Speed:%f\n", preset, (double)preset_angle, (double)actual_angle, ARM_MIN_SPEED + (ARM_MAX_SPEED - ARM_MIN_SPEED)*(temp_angle));
			
			if (actual_angle < preset_angle)
			{
				temp_angle = ((float)(preset_angle - actual_angle))/(float)ARM_RAMP_START;
				
				if(temp_angle > 1)
				{
					temp_angle = 1;
				}
				if (actual_angle > 655)
				{
					armMotor->Set((ARM_MIN_SPEED + (ARM_MAX_SPEED - ARM_MIN_SPEED)*(temp_angle)/2));
					//printf("Moving Reverse - Back Side\n");
				}
				else
				{
					armMotor->Set((ARM_MIN_SPEED + (ARM_MAX_SPEED - ARM_MIN_SPEED)*(temp_angle)));
					//printf("Moving Reverse - Front Side\n");
				}

			}	
			else
			{
				temp_angle = ((float)(actual_angle - preset_angle))/(float)ARM_RAMP_START;
									
				if(temp_angle > 1)
				{
					temp_angle = 1;
				}
				if (actual_angle > 655)
				{
					armMotor->Set(-1 * ((ARM_MIN_SPEED + (ARM_MAX_SPEED - ARM_MIN_SPEED)*(temp_angle))));
					//printf("Moving Forward - Back side\n");
				}
				else
				{
					armMotor->Set(-1 * ((ARM_MIN_SPEED + (ARM_MAX_SPEED - ARM_MIN_SPEED)*(temp_angle))/2));
					//printf("Moving Forward - Front Side\n");
				}
			}
		}
	}
}

int ThunderArm::ArmCheckOverrides()
{
	//Get out of minibot zone if mini is deployed
	if(actual_angle <= ARM_MINI_ZONE && deployer->GetMinibotDeployed())
	{
		armMotor->Set(ARM_MIN_SPEED);
		if(preset_angle < ARM_MINI_ZONE)
			preset_angle = ARM_MINI_ZONE;
		return 1;
	}
	//Get out of tusks zone if tusks are deploed
	if(actual_angle <= ARM_TUSK_ZONE && (deployer->GetTusksDeployed() || deployer->GetMinibotDeployed()))
	{
		armMotor->Set(ARM_MIN_SPEED);
		if(preset_angle < ARM_TUSK_ZONE)
			preset_angle = ARM_TUSK_ZONE;
		return 1;
	}
	//Get out of hitting the floor in front
	if((/*actual_angle < ARM_TOO_LOW_FRONT || */(armFrontLimit->Get() && !front_limit_broken)) && !start_match)
	{
		armMotor->Set(ARM_MIN_SPEED);
		if(preset_angle < ARM_TOO_LOW_FRONT)
			preset_angle = ARM_TOO_LOW_FRONT;
		return 1;
	}
	//Get out of hitting floor in back
	if (/*actual_angle > ARM_TOO_LOW_BACK ||*/ (armBackLimit->Get() && !back_limit_broken))
	{
		armMotor->Set(-ARM_MIN_SPEED);
		if(preset_angle > ARM_TOO_LOW_BACK)
			preset_angle = ARM_TOO_LOW_BACK;
		return -1;
	}
	//Get out of hitting pole when tusks and not mini are deployed
	if(actual_angle >= ARM_POLE_ZONE && deployer->GetTusksDeployed() && !deployer->GetMinibotDeployed())
	{
		armMotor->Set(-ARM_MIN_SPEED);
		if(preset_angle > ARM_POLE_ZONE)
			preset_angle = ARM_POLE_ZONE;
		return -1;
	}	
	if(actual_angle >= ARM_POST_MINI_ZONE && deployer->GetMinibotDeployed())
	{
		armMotor->Set(-ARM_MIN_SPEED);
		if(preset_angle > ARM_POST_MINI_ZONE)
			preset_angle = ARM_POST_MINI_ZONE;
		return -1;
	}
	return 0;
}

void ThunderArm::ArmGoUpDown(float speed)
{
	armSpeed = speed;
}

void ThunderArm::ArmGotoPreset(PresetPositions selection, bool inverted)
{
	invert = inverted;
	preset = selection;
	if(selection != PRE_NONE)
	{
		adhocing = false;
		if(!invert)
		{
			//UNINVERTED POSITIONS
			preset_angle = presetData[selection];
		}
		else
		{
			//INVERTED POSITIONS
			preset_angle = presetDataInverted[selection];
		}
	}
}

void ThunderArm::ArmSetAdhocPreset()
{
	if(preset == PRE_NONE && !adhocing && !adhoc_disabled)
	{
		preset_angle = actual_angle;
		adhocing = true;
	}
}

void ThunderArm::ArmStopAdhoc()
{
	adhocing = false;
}

bool ThunderArm::ArmGetAdhoc()
{
	return adhocing;
}

void ThunderArm::ArmDisableAdhoc(bool disable)
{
	adhoc_disabled = disable;
}

bool ThunderArm::ArmGetAdhocDisabled()
{
	return adhoc_disabled;
}

bool ThunderArm::ArmCheckPresetDone()
{
	return preset_done;
}

ThunderArm::PresetPositions ThunderArm::ArmCheckPreset()
{
	return preset;
}

int ThunderArm::ArmDebugAngle()
{
	if(encoder_broken)
	{
		//printf("ENCODER SET TO BROKEN");
		return 0;
	}
	
	//printf("Encoder is: %d \n", actual_angle);
	return actual_angle;
}

int ThunderArm::ArmDebugEncoderLeft()
{
	return armEncoderLeft->GetRaw();
}

int ThunderArm::ArmDebugEncoderRight()
{
	return armEncoderRight->GetRaw();
}

bool ThunderArm::ArmDebugFrontLimit()
{
	return armFrontLimit->Get() != 0;
}

bool ThunderArm::ArmDebugBackLimit()
{
	return armBackLimit->Get() != 0;
}

bool ThunderArm::ArmDebugCalibLimit()
{
	return armCalibLimit->GetVoltage() > 3;
}

void ThunderArm::ArmEncoderSwap(bool swap)
{
	encoder_swapped = swap;
}

void ThunderArm::ArmEncoderBroken(bool broken)
{
	encoder_broken = broken;
}

void ThunderArm::ArmSetStartMatch()
{
	start_match = true;
	preset = PRE_NONE;
	angle_error_Left = 0;
	angle_error_Right = 0;
	preset_angle = 0;
	
}

bool ThunderArm::ArmGetStartMatch()
{
	return start_match;
}

bool ThunderArm::ArmCheckTusksReady()
{
	if(encoder_broken)
	{
		return true;
	}
	
	if(actual_angle <= ARM_TUSK_ZONE || actual_angle >= ARM_POLE_ZONE)
	{
		return false;
	}
	return true;
}

void ThunderArm::ArmResetAfterAuto()
{
	armSpeed = 0;
	preset = PRE_NONE;
	preset_angle = 0;
	adhocing = false;
}

bool ThunderArm::ArmCheckMinibotReady()
{
	if(encoder_broken)
	{
		return true;
	}
	
	if(actual_angle <= ARM_MINI_ZONE || actual_angle >= ARM_POLE_ZONE)
	{
		return false;
	}
	return true;
}

void ThunderArm::ArmCalibLimitBroken(bool broken)
{
	calib_limit_broken = broken;
}

void ThunderArm::ArmFrontLimitBroken(bool broken)
{
	front_limit_broken = broken;
}

void ThunderArm::ArmBackLimitBroken(bool broken)
{
	back_limit_broken = broken;
}

bool ThunderArm::ArmWithinRange(float number, float range, float goal)
{
	if(number >= goal - range && number <= goal + range)
	{
		return true;
	}
	return false;
}

bool ThunderArm::GetArmInverted()
{
	return invert;
}

// return true if the arm cannot be used in autonomous
bool ThunderArm::IsArmBroken()
{
	return (encoder_broken);
}

