#include "Resources.h"
#include "ThunderAuto.h"

// the distance we want to be from the front of pegs.
// this gives us room to lift the arm for following
// the straight line and for dead reckoning. the robot
// will be at this distance after the tube has been
// placed. the robot will be at this distance at
// for all autonomous modes.
#define STOP_DISTANCE_FROM_PEG	57 

// defines for straight line
#define INITAL_STRAIGHT_DRIVE	(216 - 16 - STOP_DISTANCE_FROM_PEG)
#define STRAIGHT_TOP_PEG_DRIVE	66	// distance to drive after putting up arm
#define STRAIGHT_MID_PEG_DRIVE	36	// distance to drive after putting up arm
#define STRAIGHT_LOW_PEG_DRIVE	39	// distance to drive after putting up arm
#define STRAIGHTN_TOP_PEG_DRIVE	64	// distance to drive after putting up arm
#define STRAIGHTN_MID_PEG_DRIVE	16	// distance to drive after putting up arm
#define STRAIGHTN_LOW_PEG_DRIVE	19	// distance to drive after putting up arm

// defines for following fork 
#define INITAL_FORK_DRIVE		159	// distance to fork
#define SECOND_DRIVE_FORK		19 	// 2nd drive after fork, half distance
//#define TURN_FORK				60	// angle of line at fork
//#define TOP_THIRD_FORK_DRIVE 	60	// distance to driver after the fork
//#define MID_THIRD_FORK_DRIVE 	60	// distance to driver after the fork
//#define LOW_THIRD_FORK_DRIVE 	0	// distance to driver after the fork
//#define FORK_FUDGE_FACTOR		5	// allowed error of robot to line at fork
#define FORK_TOP_PEG_TURN		35	// amount to turn to align with peg
#define FORK_LOW_PEG_TURN		33	// amount to turn to align with peg
#define FORK_MID_PEG_TURN		39	// amount to turn to align with peg
#define FORK_TOP_PEG_DRIVE		16//24	// amount to drive to put tube over peg
#define FORK_LOW_PEG_DRIVE		0//14	// amount to drive to put tube over peg
#define FORK_MID_PEG_DRIVE		16//36	// amount to drive to put tube over peg
// 16 is the distance the pegs stick out from the wall
#define LAST_FORK_MOD			(16 + STOP_DISTANCE_FROM_PEG)

// defines for dead reckoning
#define INITAL_DEAD_DRIVE		INITAL_STRAIGHT_DRIVE
#define DEAD_TOP_PEG_DRIVE		STRAIGHTN_TOP_PEG_DRIVE-5	// distance to drive after putting up arm
#define DEAD_MID_PEG_DRIVE		STRAIGHTN_MID_PEG_DRIVE	// distance to drive after putting up arm
#define DEAD_LOW_PEG_DRIVE		STRAIGHTN_LOW_PEG_DRIVE	// distance to drive after putting up arm

// final actions
#define BACK_UP_DRIVE			-229	// amount to drive to midfield

#define ERROR_MARGIN 25 //number of degrees of error to allow when driving

ThunderAuto::ThunderAuto(ThunderDrive *drive, ThunderArm *arm, ThunderManip *manip)
{
	ThunderDriveCode = drive;
	ThunderArmCode = arm; 
	ThunderManipCode = manip;
	logger = new UDPLogger();
	logger->init(1140);
	
	Initialize(0);
}

ThunderAuto::~ThunderAuto()
{
	delete logger;
}

void ThunderAuto::Initialize(float delayTimeargue)
{
	// initialize states
	straight_state = ThunderAuto::delay;
	fork_state = ThunderAuto::fork_delay;
	dead_state = ThunderAuto::dead_delay;
	subphase = ThunderAuto::raise_arm_start;
	delayTime = delayTimeargue;
}

void ThunderAuto::run_auto (Approach approach, Peg_Height peg_height,
							Ending_Action ending_action)
{
	bool done = false;
	
	switch (approach)
	{
		default:
		case no_approach: // 0
			endAuto(true);
			break;
			
		case straight_left: // 1
		//case straight_right: // 2
			done = straight(peg_height);
			break;
		case straight_right: // 2
			done = straightnew(peg_height);
			break;
			
		case fork_left: // 3
			done = fork (false, peg_height);
			break;
			
		case fork_right: // 4
			done = fork (true, peg_height);
			break;

		case dead_reck_left_norm: // 5
		//case dead_reck_right_normal: // 7
			done = dead_reckoning (peg_height, false);
			break;

		case dead_reck_left_offset: // 6
		//case dead_reck_right_offset: // 8
			done = dead_reckoning (peg_height, true);
			break;
			
		//case straight_right: // 2
		case dead_reck_right_normal: // 7
		case dead_reck_right_offset: // 8
			endAuto(true);
			printf ("You have a failure: Invalid Command");
			break;
	}

	// do ending actions after tube is on peg
	if (done)
	{
		switch (ending_action)
		{
			default:
			case no_end:
				endAuto(true);
				break;
			
			case raise_arm:
				End_Action(false, false);
				break;
			
			case turn_around:
				//End_Action(false, true);
				End_Action(false, false);
				break;
			
			case drive_back:
				//End_Action (true, false);
				End_Action(false, false);
				break;
			
			case turn_back:
				End_Action(false, false);
				//End_Action (true, true);
				break;
				
			case go_left_lane_no_turn:
			case go_left_lane_turn:
			case go_right_lane_no_turn:
			case go_right_lane_turn:
				endAuto(true);
				printf ("You have a failure: Invalid Command");
				break;
	
		}//switch
	}//if
}


void ThunderAuto::do_nothing()
{
	ThunderDriveCode->Drive_Stop_Auto_Task();
}

bool ThunderAuto::straight (Peg_Height peg_height)
{
	switch (straight_state)
	{
		case ThunderAuto::delay:
			begin_time = GetTime();
			logger->log("Start Delay State for %f seconds\n", (double)delayTime);
			straight_state = ThunderAuto::delay_check;
			break;
			
		case ThunderAuto::delay_check:
			double curTime = GetTime();
			if (curTime >= (begin_time + (double)delayTime))
			{
				logger->log("Delay State Complete\n");
				straight_state = ThunderAuto::drive_forward;
				ThunderDriveCode->Drive_Clear_Gyro();
				Wait(0.025);
			}                       
			break;
			
		case ThunderAuto::drive_forward:
			if (peg_height != no_peg)
			{	
				//ThunderDriveCode->Drive_Clear_Gyro();
				ThunderDriveCode->Drive_Keep_On_Line(INITAL_STRAIGHT_DRIVE);
				ThunderArmCode->ArmSetStartMatch();
				ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_MEDIUM_OFFSET, false);
				updateAngle(0);
				straight_state = ThunderAuto::drive_forward_check;
				logger->log("Driving Forward\n");
			}
			else
			{
				endAuto(true);
			}
			break;

		case ThunderAuto::drive_forward_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done()
			   && !ThunderArmCode->ArmGetStartMatch())
			{
				logger->log("Done Driving Forward\n");
				straight_state = ThunderAuto::twist;
			}
			else
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::twist:
			if (peg_height == ThunderAuto::top)
			{
				ThunderManipCode->ManipTwistAuto (ThunderManip::HIGH_OFFSET);
				ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_HIGH_OFFSET, false);
			}
			else if (peg_height == ThunderAuto::low)
			{
				ThunderManipCode->ManipTwistAuto (ThunderManip::LOW_OFFSET);
				ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_LOW_OFFSET, false);
			}
			else
			{
				ThunderManipCode->ManipTwistAuto (ThunderManip::MEDIUM_OFFSET);
				ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_MEDIUM_OFFSET, false);
			}
			logger->log("Moving arm and manipulator\n");
			straight_state = ThunderAuto::twist_check; 
			break;
	
		case ThunderAuto::twist_check:
			if (ThunderArmCode->ArmCheckPresetDone()
			    && ThunderManipCode->ManipCheckAutoDone())
			{
				logger->log("Done Moving arm and manipulator\n");
				straight_state = ThunderAuto::drive_tube_peg;
			}
			else
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::drive_tube_peg:
			logger->log("Driving forward to place tube\n");
			// note that this step will continue without checking to see if the
			// robot is centered on the line; may (with probability < 10%) be dangerous
			if (peg_height == ThunderAuto::top)
			{
				ThunderDriveCode->Drive_Keep_On_Line(STRAIGHT_TOP_PEG_DRIVE,false);	
			}
			else if (peg_height==ThunderAuto::low)
			{
				ThunderDriveCode->Drive_Keep_On_Line(STRAIGHT_LOW_PEG_DRIVE,false);
			}
			else
			{
				ThunderDriveCode->Drive_Keep_On_Line(STRAIGHT_MID_PEG_DRIVE,false);
			}
			updateAngle(0);
			straight_state = ThunderAuto::drive_tube_peg_check;
			break;
			
		case ThunderAuto::drive_tube_peg_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done())
			{
				logger->log("Done driving forward to place tube\n");
				straight_state = ThunderAuto::ejection;
			}
			else
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::ejection:
			logger->log("Ejecting the tube\n");
			ThunderManipCode->ManipOutAuto();
			straight_state = ThunderAuto::ejection_check;
			break;
			
		case ThunderAuto::ejection_check:
			if (ThunderManipCode->ManipCheckAutoDone())
			{
				logger->log("Done Ejecting the tube\n");
				straight_state = ThunderAuto::reverse;
			}
			else 
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::reverse:
			logger->log("Driving away from the pegs\n");
			if (peg_height == ThunderAuto::top)
			{
				ThunderDriveCode->Drive_Forward_Distance(-STRAIGHT_TOP_PEG_DRIVE);
			}
			else if (peg_height==ThunderAuto::low)
			{
				ThunderDriveCode->Drive_Forward_Distance(-STRAIGHT_LOW_PEG_DRIVE);
			}
			else
			{
				ThunderDriveCode->Drive_Forward_Distance(-STRAIGHT_MID_PEG_DRIVE);
			}
			updateAngle(0);
			straight_state = ThunderAuto::reverse_check;
			break;
			
		case ThunderAuto::reverse_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done())
			{
				logger->log("Done Driving away from the pegs\n");
				straight_state = ThunderAuto::done;
			}
			else 
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::done:
			//printf ("We are done with placing the tube\n");
			return (true);
			
		case ThunderAuto::error:
			//printf ("We are on ice, as apposed to fire.\n");
			break;
	}//switch
	return (false);
}

bool ThunderAuto::straightnew (Peg_Height peg_height)
{
	switch (straight_state)
	{
		case ThunderAuto::delay:
			begin_time = GetTime();
			logger->log("Start Delay State for %f seconds\n", (double)delayTime);
			straight_state = ThunderAuto::delay_check;
			break;
			
		case ThunderAuto::delay_check:
			double curTime = GetTime();
			if (curTime >= (begin_time + (double)delayTime))
			{
				logger->log("Delay State Complete\n");
				straight_state = ThunderAuto::drive_forward;
				ThunderDriveCode->Drive_Clear_Gyro();
				Wait(0.025);
			}                       
			break;
			
		case ThunderAuto::drive_forward:
			if (peg_height != no_peg)
			{	
				//ThunderDriveCode->Drive_Clear_Gyro();
			//	ThunderDriveCode->Drive_Keep_On_Line(INITAL_STRAIGHT_DRIVE);
				if (peg_height == ThunderAuto::top)
				{
					ThunderDriveCode->Drive_Keep_On_Line(INITAL_STRAIGHT_DRIVE+STRAIGHTN_TOP_PEG_DRIVE,false);	
				}
				else if (peg_height==ThunderAuto::low)
				{
					ThunderDriveCode->Drive_Keep_On_Line(INITAL_STRAIGHT_DRIVE+STRAIGHTN_LOW_PEG_DRIVE,false);
				}
				else
				{
					ThunderDriveCode->Drive_Keep_On_Line(INITAL_STRAIGHT_DRIVE+STRAIGHTN_MID_PEG_DRIVE,false);
				}
				ThunderArmCode->ArmSetStartMatch();
				if (peg_height == ThunderAuto::top)
				{
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_HIGH_OFFSET, false);
				}
				else if (peg_height == ThunderAuto::low)
				{
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_LOW_OFFSET, false);
				}
				else
				{
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_MED_OFFSET, false);
				}
				updateAngle(0);
				straight_state = ThunderAuto::drive_forward_check;
				logger->log("Driving Forward\n");
			}
			else
			{
				endAuto(true);
			}
			break;

		case ThunderAuto::drive_forward_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done()
			   && !ThunderArmCode->ArmGetStartMatch())
			{
				logger->log("Done Driving Forward\n");
				straight_state = ThunderAuto::twist;
			}
			else
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::twist:
			if (peg_height == ThunderAuto::top)
			{
				//ThunderManipCode->ManipTwistAuto (ThunderManip::HIGH_OFFSET);
				ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_HIGH_OFFSET, false);
			}
			else if (peg_height == ThunderAuto::low)
			{
				//ThunderManipCode->ManipTwistAuto (ThunderManip::LOW_OFFSET);
				ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_LOW_OFFSET, false);
			}
			else
			{
				//ThunderManipCode->ManipTwistAuto (ThunderManip::MEDIUM_OFFSET);
				ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_MED_OFFSET, false);
			}
			logger->log("Moving arm\n");// and manipulator\n");
			straight_state = ThunderAuto::twist_check; 
			break;
	
		case ThunderAuto::twist_check:
			if (ThunderArmCode->ArmCheckPresetDone()
			    /*&& ThunderManipCode->ManipCheckAutoDone()*/)
			{
				logger->log("Done Moving arm\n");// and manipulator\n");
				straight_state = ThunderAuto::drive_tube_peg_check;
			}
			else
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::drive_tube_peg:
			logger->log("Driving forward to place tube\n");
			// note that this step will continue without checking to see if the
			// robot is centered on the line; may (with probability < 10%) be dangerous
			if (peg_height == ThunderAuto::top)
			{
				ThunderDriveCode->Drive_Keep_On_Line(STRAIGHTN_TOP_PEG_DRIVE,false);	
			}
			else if (peg_height==ThunderAuto::low)
			{
				ThunderDriveCode->Drive_Keep_On_Line(STRAIGHTN_LOW_PEG_DRIVE,false);
			}
			else
			{
				ThunderDriveCode->Drive_Keep_On_Line(STRAIGHTN_MID_PEG_DRIVE,false);
			}
			updateAngle(0);
			straight_state = ThunderAuto::drive_tube_peg_check;
			break;
			
		case ThunderAuto::drive_tube_peg_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done())
			{
				logger->log("Done driving forward to place tube\n");
				straight_state = ThunderAuto::ejection;
			}
			else
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::ejection:
			logger->log("Ejecting the tube and lowering arm\n");
			ThunderManipCode->ManipOutAuto();
			if (peg_height == ThunderAuto::top)
			{
				ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_HIGH_OFFSET_LOWER, false);
			}
			else if (peg_height == ThunderAuto::low)
			{
				ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_LOW_OFFSET_LOWER, false);
			}
			else
			{
				ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_MED_OFFSET_LOWER, false);
			}
			straight_state = ThunderAuto::ejection_check;
			break;
			
		case ThunderAuto::ejection_check:
			//if (ThunderManipCode->ManipCheckAutoDone())		
			if (ThunderArmCode->ArmCheckPresetDone())
			{
				logger->log ("Done lowering arm while ejecting the tube\n");//Ejecting the tube\n");
				straight_state = ThunderAuto::reverse;
			}
			else 
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::reverse:
			logger->log ("Driving away from the pegs\n");
			if (peg_height == ThunderAuto::top)
			{
				ThunderDriveCode->Drive_Forward_Distance(-STRAIGHTN_TOP_PEG_DRIVE);
			}
			else if (peg_height==ThunderAuto::low)
			{
				ThunderDriveCode->Drive_Forward_Distance(-STRAIGHTN_LOW_PEG_DRIVE);
			}
			else
			{
				ThunderDriveCode->Drive_Forward_Distance(-STRAIGHTN_MID_PEG_DRIVE);
			}
			updateAngle(0);
			straight_state = ThunderAuto::reverse_check;
			break;
			
		case ThunderAuto::reverse_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done())
			{
				logger->log ("Done Driving away from the pegs\n");
				straight_state = ThunderAuto::done;
			}
			else 
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::done:
			//printf ("We are done with placing the tube\n");
			return (true);
			
		case ThunderAuto::error:
			//printf ("We are on ice, as apposed to fire.\n");
			break;
	}//switch
	return (false);
}
	
	
	
bool ThunderAuto::fork(bool right_direction, Peg_Height peg_height)
{
	switch (fork_state)
	{
	case ThunderAuto::fork_delay: 
		begin_time = GetTime();
		logger->log("Start Delay State for %f seconds\n", (double)delayTime);
		fork_state = ThunderAuto::fork_delay_check;
		break;
		
	case ThunderAuto::fork_delay_check:
		double curTime = GetTime();
		if (curTime >= (begin_time + (double)delayTime))
		{
			logger->log("Delay State Complete\n");
			// resetting the gyro to 0 in case it is too close to 300 degress
			// thunderdrive doesn't try to move out of range
			fork_state = ThunderAuto::drive_forward_fork;
			ThunderDriveCode->Drive_Clear_Gyro();
			Wait(0.025);
		}                       
		break;
	
		case ThunderAuto::drive_forward_fork:
			if (peg_height != no_peg)
			{	
				logger->log ("Driving to fork.\n");
				ThunderDriveCode->Drive_Keep_On_Line(INITAL_FORK_DRIVE);
				ThunderArmCode->ArmSetStartMatch();
				if (peg_height == ThunderAuto::top)
				{
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_HIGH, false);
				}
				else if (peg_height == ThunderAuto::low)
				{
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_LOW, false);
				}
				else
				{
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_MED, false);
				}
				updateAngle(0);
				fork_state = ThunderAuto::drive_forward_fork_check;
			}
			else
			{
				endAuto(true);
			}
			break;
			
		case ThunderAuto::drive_forward_fork_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done()
				&&	!ThunderArmCode->ArmGetStartMatch())
			{
				logger->log ("We are now at the fork.\n");
				fork_state = ThunderAuto::ejection_prep;
			}
			else
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::ejection_prep:
			logger->log ("Moving arm to selected height\n");
			if (peg_height == ThunderAuto::top)
			{
				//ThunderManipCode->ManipTwistAuto (ThunderManip::HIGH);
				ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_HIGH, false);
			}
			else if (peg_height == ThunderAuto::low)
			{
				//ThunderManipCode->ManipTwistAuto (ThunderManip::LOW);
				ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_LOW, false);
			}
			else
			{
				//ThunderManipCode->ManipTwistAuto (ThunderManip::MEDIUM);
				ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_MED, false);
			}
			fork_state = ThunderAuto::ejection_prep_check;
			break;
			
		case ThunderAuto::ejection_prep_check:
			if (ThunderArmCode->ArmCheckPresetDone()
				/*&& ThunderManipCode->ManipCheckAutoDone()*/)
			{
				logger->log ("Done moving arm\n");
				fork_state = ThunderAuto::second_drive;	
			}
			else
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::second_drive:
			logger->log ("Just a little more....\n");
			ThunderDriveCode->Drive_Forward_Distance(SECOND_DRIVE_FORK);
			fork_state = ThunderAuto::second_drive_check;
			updateAngle (0);
			break;
			
		case ThunderAuto::second_drive_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done())
			{	
				logger->log ("Now we are done doing a little!\n");
				fork_state = ThunderAuto::little_turn;
			}
			else 
			{
				checkCourse(); 
			}	
			break;	 

			/*
		case ThunderAuto::turn_fork:
			if (peg_height == ThunderAuto::low)
			{
				fork_state = ThunderAuto::little_turn;
				break;
			}
			if (right_direction)
			{	
				printf ("Turning to the right\n");
				ThunderDriveCode->Drive_Turn_Until_Line(TURN_FORK + FORK_FUDGE_FACTOR);
				updateAngle(TURN_FORK + FORK_FUDGE_FACTOR);
			}
			else
			{	
				printf ("Turning to  the left\n");
				ThunderDriveCode->Drive_Turn_Until_Line(-(TURN_FORK + FORK_FUDGE_FACTOR));
				updateAngle(-(TURN_FORK + FORK_FUDGE_FACTOR));
			}
			fork_state = ThunderAuto::turn_fork_check;
			break;
			
		case ThunderAuto::turn_fork_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done())
			{	
				printf ("We are done turning\n");
				fork_state = ThunderAuto::thrid_drive;
			}
			else
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::thrid_drive:
			printf ("Moving down the line\n");
			if (peg_height == ThunderAuto::top)
			{
				ThunderDriveCode->Drive_Keep_On_Line(TOP_THIRD_FORK_DRIVE,false);
			}
			else if (peg_height == ThunderAuto::low)
			{
				ThunderDriveCode->Drive_Keep_On_Line(LOW_THIRD_FORK_DRIVE,false);
			}
			else
			{
				ThunderDriveCode->Drive_Keep_On_Line(MID_THIRD_FORK_DRIVE,false);
			}
			fork_state = ThunderAuto::third_drive_check;
			updateAngle(0);
			break;
			
		case ThunderAuto::third_drive_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done())
			{
				fork_state = ThunderAuto::little_turn;
				printf ("We are done moving down the line\n");
			}
			else
			{
				checkCourse();
			}
			break;*/
		
		case ThunderAuto::little_turn:
			logger->log ("Aligning robot to pegs\n");
			
			if (right_direction)
			{
				if (peg_height == ThunderAuto::top)
				{
					ThunderDriveCode->Drive_Turn_Angle(FORK_TOP_PEG_TURN);	
					updateAngle(FORK_TOP_PEG_TURN);
				}
				
				else if (peg_height==ThunderAuto::low)
				{
					ThunderDriveCode->Drive_Turn_Angle(FORK_LOW_PEG_TURN);
					updateAngle(FORK_LOW_PEG_TURN);
				}
				else
				{
					ThunderDriveCode->Drive_Turn_Angle(FORK_MID_PEG_TURN);
					updateAngle(FORK_MID_PEG_TURN);
				}
			}
			else 
			{	
				if (peg_height == ThunderAuto::top)
				
				{
					ThunderDriveCode->Drive_Turn_Angle(-FORK_TOP_PEG_TURN);	
					updateAngle(-FORK_TOP_PEG_TURN);
				}
				
				else if (peg_height==ThunderAuto::low)
				{
					ThunderDriveCode->Drive_Turn_Angle(-FORK_LOW_PEG_TURN);
					updateAngle(-FORK_LOW_PEG_TURN);
				}
				else
				{
					ThunderDriveCode->Drive_Turn_Angle(-FORK_MID_PEG_TURN);
					updateAngle(-FORK_MID_PEG_TURN);
				}
			}
			fork_state = ThunderAuto::little_turn_check;
			break;
		
		case ThunderAuto::little_turn_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done())
			{
				logger->log ("We are aligned with the pegs\n");
				fork_state = ThunderAuto::drive_peg;
			}
			else
			{
				checkCourse ();
			}
			break;
		
		case ThunderAuto::drive_peg:
			logger->log ("Driving tube onto peg\n");
			if (peg_height == ThunderAuto::top)
			{
				ThunderDriveCode->Drive_Forward_Distance(FORK_TOP_PEG_DRIVE);
			}
			else if (peg_height == ThunderAuto::low)
			{
				ThunderDriveCode->Drive_Forward_Distance(FORK_LOW_PEG_DRIVE);
			}
			else
			{
				ThunderDriveCode->Drive_Forward_Distance(FORK_MID_PEG_DRIVE);
			}
			updateAngle(0);
			fork_state = ThunderAuto::drive_peg_check;
			break;
			
		case ThunderAuto::drive_peg_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done())
			{
				logger->log ("The tube is on the peg\n");
				fork_state = ThunderAuto::fork_ejection;
			}
			else 
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::fork_ejection:
			logger->log ("Ejecting tube and lowering arm...\n");
			if (peg_height == ThunderAuto::top)
			{
				ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_HIGH_LOWER, false);
			}
			else if (peg_height == ThunderAuto::low)
			{
				ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_LOW_LOWER, false);
			}
			else
			{
				ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_MED_LOWER, false);
			}
			ThunderManipCode->ManipOutAuto();
			fork_state = ThunderAuto::fork_ejection_check;
			break;
			
		case ThunderAuto::fork_ejection_check:
			//if (ThunderManipCode->ManipCheckAutoDone())
			if (ThunderArmCode->ArmCheckPresetDone())
			{
				logger->log ("Done lowering arm while ejecting tube\n");
				fork_state = ThunderAuto::drive_fork_back;
			}
			break;
			
		case ThunderAuto::drive_fork_back:
			logger->log ("Driving backwards....\n");
			if (peg_height == ThunderAuto::top)
			{
				ThunderDriveCode->Drive_Forward_Distance(-FORK_TOP_PEG_DRIVE);
			}
			else if (peg_height == ThunderAuto::low)
			{
				ThunderDriveCode->Drive_Forward_Distance(-FORK_LOW_PEG_DRIVE);
			}
			else
			{
				ThunderDriveCode->Drive_Forward_Distance(-FORK_MID_PEG_DRIVE);
			}
			updateAngle(0);
			fork_state = ThunderAuto::drive_back_fork_check;
			break;
		
		case ThunderAuto::drive_back_fork_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done())
			{
				logger->log ("We are done driving backwards\n");
				fork_state = ThunderAuto::turn;
			}
			else 
			{
				checkCourse();
			}
			break;	
		
		case ThunderAuto::turn:
			logger->log ("Positioning for ending action\n");
			/*if (peg_height == ThunderAuto::top)
			{
				ThunderDriveCode->Drive_Turn_Angle(-FORK_TOP_PEG_TURN);
				updateAngle(-FORK_TOP_PEG_TURN);
			}
			else if (peg_height == ThunderAuto::low)
			{
				ThunderDriveCode->Drive_Turn_Angle(-FORK_LOW_PEG_TURN);
				updateAngle(-FORK_LOW_PEG_TURN);
			}
			else
			{
				ThunderDriveCode->Drive_Turn_Angle(-FORK_MID_PEG_TURN);
				updateAngle(-FORK_MID_PEG_TURN);
			}*/
			ThunderDriveCode->Drive_Turn_Angle(-ThunderDriveCode->Get_Gyro_Angle());	
			updateAngle(-ThunderDriveCode->Get_Gyro_Angle());
			fork_state = ThunderAuto::turn_check;
			break;
		
		case ThunderAuto::turn_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done())
			{
				logger->log ("We are now perpendicular to the wall\n");
				fork_state = ThunderAuto::last_fork_drive;
			}
			else 
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::last_fork_drive:
			logger->log ("One last drive\n");
			if (peg_height == ThunderAuto::top)
			{
				ThunderDriveCode->Drive_Forward_Distance(-(LAST_FORK_MOD-FORK_TOP_PEG_DRIVE));
			}
			else if (peg_height == ThunderAuto::low)
			{
				ThunderDriveCode->Drive_Forward_Distance(-(LAST_FORK_MOD-FORK_LOW_PEG_DRIVE));
			}
			else
			{
				ThunderDriveCode->Drive_Forward_Distance(-(LAST_FORK_MOD-FORK_MID_PEG_DRIVE));
			}
			updateAngle(0);
			fork_state = ThunderAuto::last_fork_fork_drive_check;
			break;
	
		case ThunderAuto::last_fork_fork_drive_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done())
			{
				logger->log ("We are done driving\n");
				fork_state = ThunderAuto::fork_done;
			}
			else 
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::fork_done:
			//printf ("Entering Final AutoPhase\n");
			return (true);
			
		case ThunderAuto::fork_error:
			//printf ("You have been backstabbed by physics\n");
			break;
	} // switch
	
	return (false);
}

bool ThunderAuto::dead_reckoning (Peg_Height peg_height, bool offset)
{
	switch (dead_state)
	{
	case ThunderAuto::dead_delay: 
			begin_time = GetTime();
			logger->log("Start Delay State for %f seconds\n", (double)delayTime);
			dead_state = ThunderAuto::dead_delay_check;
			break;
			
		case ThunderAuto::dead_delay_check:
			double curTime = GetTime();
			if (curTime >= (begin_time + (double)delayTime))
			{
				logger->log("Delay State Complete\n");
				// resetting the gyro to 0 in case it is too close to 300 degress
				// thunderdrive doesn't try to move out of range
				dead_state = ThunderAuto::dead_drive_forward;
				ThunderDriveCode->Drive_Clear_Gyro();
				Wait(0.025);
			}                       
			break;
		case ThunderAuto::dead_drive_forward:
			if (peg_height != no_peg)
			{	
				//ThunderDriveCode->Drive_Clear_Gyro();
				//ThunderDriveCode->Drive_Forward_Distance(INITAL_DEAD_DRIVE);
				if (peg_height == ThunderAuto::top)
				{
					ThunderDriveCode->Drive_Forward_Distance(INITAL_DEAD_DRIVE+DEAD_TOP_PEG_DRIVE);
				}
				else if (peg_height == ThunderAuto::low)
				{
					ThunderDriveCode->Drive_Forward_Distance(INITAL_DEAD_DRIVE+DEAD_LOW_PEG_DRIVE);
				}
				else
				{
					ThunderDriveCode->Drive_Forward_Distance(INITAL_DEAD_DRIVE+DEAD_MID_PEG_DRIVE);
				}
				logger->log ("Driving Forward\n");
				ThunderArmCode->ArmSetStartMatch();
				if (offset)
				{
					if (peg_height == ThunderAuto::top)
					{
						ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_HIGH_OFFSET, false);
					}
					else if (peg_height == ThunderAuto::low)
					{
						ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_LOW_OFFSET, false);
					}
					else
					{
						ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_MED_OFFSET, false);
					}
				}
				else
				{
					if (peg_height == ThunderAuto::top)
					{
						ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_HIGH, false);
					}
					else if (peg_height == ThunderAuto::low)
					{
						ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_LOW, false);
					}
					else
					{
						ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_MED, false);
					}
				}
				updateAngle(0);
				dead_state = dead_drive_forward_check;
			}
			else
			{
				endAuto(true);
			}
			break;
			
		case ThunderAuto::dead_drive_forward_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done()
			   && !ThunderArmCode->ArmGetStartMatch())
			{
				logger->log ("Finished Driving Forward\n");
				dead_state = ThunderAuto::dead_twist;
			}
			else
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::dead_twist:
			if (offset)
			{
				logger->log ("Moving arm (offset).\n");// and manipulator (offset).\n");
				if (peg_height == ThunderAuto::top)
				{
					//ThunderManipCode->ManipTwistAuto (ThunderManip::HIGH_OFFSET);
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_HIGH_OFFSET, false);
				}
				else if (peg_height == ThunderAuto::low)
				{
					//ThunderManipCode->ManipTwistAuto (ThunderManip::LOW_OFFSET);
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_LOW_OFFSET, false);
				}
				else
				{
					//ThunderManipCode->ManipTwistAuto (ThunderManip::MEDIUM_OFFSET);
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_MED_OFFSET, false);
				}	
			}
			else
			{
				logger->log ("Moving arm (normal).\n");// and manipulator (normal).\n");
				if (peg_height == ThunderAuto::top)
				{
					//ThunderManipCode->ManipTwistAuto (ThunderManip::HIGH);
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_HIGH, false);
				}
				else if (peg_height == ThunderAuto::low)
				{
					//ThunderManipCode->ManipTwistAuto (ThunderManip::LOW);
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_LOW, false);
				}
				else
				{
					//ThunderManipCode->ManipTwistAuto (ThunderManip::MEDIUM);
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_MED, false);
				}
			}
			dead_state = ThunderAuto::dead_twist_check; 
			break;
	
		case ThunderAuto::dead_twist_check:
			if (ThunderArmCode->ArmCheckPresetDone()
			    /*&& ThunderManipCode->ManipCheckAutoDone()*/)
			{
				logger->log ("Done moving the arm\n");
				dead_state = ThunderAuto::dead_drive_tube_peg_check;
			}
			else
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::dead_drive_tube_peg:
			logger->log ("We are driving the tube on the peg\n");
			if (peg_height == ThunderAuto::top)
			{
				ThunderDriveCode->Drive_Forward_Distance(DEAD_TOP_PEG_DRIVE);	
			}
			else if (peg_height==ThunderAuto::low)
			{
				ThunderDriveCode->Drive_Forward_Distance(DEAD_LOW_PEG_DRIVE);
			}
			else
			{
				ThunderDriveCode->Drive_Forward_Distance(DEAD_MID_PEG_DRIVE);
			}
			updateAngle(0);
			dead_state = ThunderAuto::dead_drive_tube_peg_check;
			break;
			
		case ThunderAuto::dead_drive_tube_peg_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done())
			{
				logger->log ("We are done driving the tube on the peg\n");
				dead_state = ThunderAuto::dead_ejection;
			}
			else
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::dead_ejection:
			logger->log ("Ejecting tube and lowering arm\n");
			if (offset)
			{
				if (peg_height == ThunderAuto::top)
				{
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_HIGH_OFFSET_LOWER, false);
				}
				else if (peg_height == ThunderAuto::low)
				{
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_LOW_OFFSET_LOWER, false);
				}
				else
				{
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_MED_OFFSET_LOWER, false);
				}
			}
			else
			{
				if (peg_height == ThunderAuto::top)
				{
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_HIGH_LOWER, false);
				}
				else if (peg_height == ThunderAuto::low)
				{
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_LOW_LOWER, false);
				}
				else
				{
					ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_AUTO_MED_LOWER, false);
				}
			}
			ThunderManipCode->ManipOutAuto();
			dead_state = ThunderAuto::dead_ejection_check;
			break;
			
		case ThunderAuto::dead_ejection_check:
			//if (ThunderManipCode->ManipCheckAutoDone())
			if (ThunderArmCode->ArmCheckPresetDone())
			{
				logger->log ("Done lowering arm while Ejecting\n");
				dead_state = ThunderAuto::dead_reverse;
			}
			else 
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::dead_reverse:
			logger->log ("Backing away from the pegs\n");
			if (peg_height == ThunderAuto::top)
			{
				ThunderDriveCode->Drive_Forward_Distance(-DEAD_TOP_PEG_DRIVE);
			}
			else if (peg_height==ThunderAuto::low)
			{
				ThunderDriveCode->Drive_Forward_Distance(-DEAD_LOW_PEG_DRIVE);
			}
			else
			{
				ThunderDriveCode->Drive_Forward_Distance(-DEAD_MID_PEG_DRIVE);
			}
			updateAngle(0);
			dead_state = ThunderAuto::dead_reverse_check;
			break;
			
		case ThunderAuto::dead_reverse_check:
			if (ThunderDriveCode->Drive_Current_Auto_Task_Done())
			{
				logger->log ("Done Driving backwards\n");
				dead_state = ThunderAuto::dead_done;
			}
			else 
			{
				checkCourse();
			}
			break;
			
		case ThunderAuto::dead_done:
			//printf ("We are going to final actions\n");
			return (true);
		
		case ThunderAuto::undead_error:
			//printf ("You just lost the game.\n");
			break;
	}		
	return (false);
}

void ThunderAuto::updateAngle(float desiredAngle)
{
	float currentAngle = ThunderDriveCode->Get_Gyro_Angle();
	logger->log("Updating angle tolerance based on cur %f + desired %f\n", (double)currentAngle, (double)desiredAngle);

	float newAngle = currentAngle + desiredAngle;
	if (newAngle >= currentAngle)
	{
		minAngle = currentAngle - ERROR_MARGIN;
		maxAngle = newAngle + ERROR_MARGIN;
	}
	else
	{
		minAngle = newAngle - ERROR_MARGIN;
		maxAngle = currentAngle + ERROR_MARGIN;
	}

}
bool ThunderAuto::checkCourse()
{
	//temporary disable function
	//return true;
	float currentAngle = ThunderDriveCode->Get_Gyro_Angle();
	if ((currentAngle >= minAngle) && (currentAngle <= maxAngle))
	{
		return true;
	}
	else
	{
		endAuto(false);
		logger->log("Ended because %f outside %f -> %f\n", (double)currentAngle, (double)minAngle, (double)maxAngle);
		return false;
	}
}
	
void ThunderAuto::End_Action (bool drive, bool turn)
{
	switch(subphase)
	{
		case ThunderAuto::raise_arm_start:
			logger->log ("Raising arm to top peg height\n");
			ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_STRAIGHT_UP, false);
			updateAngle(0);
			subphase = ThunderAuto::raise_arm_stop;
			break;
	
		case ThunderAuto::raise_arm_stop:
			if (ThunderArmCode->ArmCheckPresetDone())
			{
				logger->log ("Done raising arm\n");
				subphase = ThunderAuto::drive_back_start; 
			}
			else
			{
				checkCourse ();
			}
			break;
		
		case ThunderAuto::drive_back_start:
			if(drive)
			{
				logger->log ("Driving to midfield\n");
				ThunderDriveCode->Drive_Forward_Distance(BACK_UP_DRIVE);
				subphase = ThunderAuto::drive_back_stop;
				updateAngle (0);
			}
			else
			{
				subphase = ThunderAuto::turn_start;
			}
			break;
		
		case ThunderAuto::drive_back_stop:
			if(ThunderDriveCode->Drive_Current_Auto_Task_Done())
			{
				logger->log ("Driving to midfield done\n");
				subphase = ThunderAuto::turn_start;
			}
			else 
			{
				checkCourse ();
			}
			break;
			
		case ThunderAuto::turn_start:
			if(turn)
			{
				ThunderDriveCode->Drive_Turn_Angle(180);
				updateAngle (180);
				logger->log ("Turning 180 degrees\n");
				subphase = ThunderAuto::turn_stop;
			}
			else
			{
				subphase = ThunderAuto::finish;
			}
			break;
			
		case ThunderAuto::turn_stop:
			if(ThunderDriveCode->Drive_Current_Auto_Task_Done())
			{
				logger->log ("Done turning 180 degrees\n");
				subphase = ThunderAuto::finish;
			}
			else
			{
				checkCourse ();
			}
			break;
			
		case ThunderAuto::finish:
			logger->log ("Done with Final Actions\n");
			
			subphase = ThunderAuto::ended;
			break;
				
		case ThunderAuto::ended:
			endAuto (true);
			break;

		case ThunderAuto::end_error: 
			logger->log ("Our ending actions have failed");
			break;
			
	}//switch
}

void ThunderAuto::endAuto(bool good_boy) 
{
	if (good_boy)
	{
		straight_state = ThunderAuto::done;
		fork_state = ThunderAuto::fork_done;
		dead_state = ThunderAuto::dead_done;
		subphase = ThunderAuto::ended;
	}
	else
	{
		straight_state = ThunderAuto::error;
		fork_state = ThunderAuto::fork_error;
		dead_state = ThunderAuto::undead_error;
		subphase = ThunderAuto::end_error;
		logger->log ("Ending Autonomous--EROR\n");
	}
	//printf ("Ending Autonomous\n");
	ThunderDriveCode->Drive_Left_Right_Motors(0,0);
	ThunderDriveCode->Drive_Stop_Auto_Task();
	ThunderArmCode->ArmGotoPreset(ThunderArm::PRE_NONE, false);	
}
