#include "Resources.h"
#include "ThunderDrive.h"
#include "ThunderArm.h"
#include "ThunderManip.h"
#include "UDPLogger.h"
#ifndef THUNDER_AUTO_H
#define THUNDER_AUTO_H

 //high level autonomous
class ThunderAuto 
{

public: 
	ThunderAuto(ThunderDrive *drive, ThunderArm *arm, ThunderManip *manip);
	~ThunderAuto();

	//the diffrent ways to approach the pegs.
	typedef enum {no_approach = 0, straight_left, straight_right,  fork_left, fork_right,
		dead_reck_left_norm, dead_reck_left_offset, dead_reck_right_normal, 
					 dead_reck_right_offset}  Approach;
	//the diffrent heights to place the peg at.
	typedef enum {no_peg = 0, top, middle, low} Peg_Height;
	
	//The diffrent ending actions to do after tube is placed.
	typedef enum {no_end = 0, raise_arm, turn_around, drive_back, turn_back,
		go_left_lane_no_turn, go_left_lane_turn, go_right_lane_no_turn,
		go_right_lane_turn} 			 Ending_Action;
		 
	// initialize autonomous 
	void Initialize(float delayTimeargue);

	//the autonomous "stages" to insert numbers for.
	void run_auto (Approach approach, Peg_Height peg_height, Ending_Action
																	ending_action);
	
private:
	double begin_time;
	float delayTime;
	
	//Set the engines from lightspeed....to stop!
	void do_nothing ();
	//drive striaght following the line and score at the peg height
	//peg_height: what height to use once at the pegs
	bool straight (Peg_Height peg_height);
	bool straightnew (Peg_Height peg_height);
	//approach the center line with a fork in the middle and scores at the peg height
	//Bool direction: what direction to go on the center fork
	bool fork (bool right_direction, Peg_Height peg_height);
	//use our expert math skills and not the lines to approach the pegs (assuming we
	//are facing the pegs directly.) and score at the peg height.
	//bool offset:Wheather or not the peg heights are the offset (if they are the 
	//higher pegs, use true) in dead reck or not.
	bool dead_reckoning (Peg_Height peg_height, bool offset);

	// stops everything!
	void endAuto(bool good_boy);

	// updates what the max and min angles will be depending on what your desired angle is
	void updateAngle(float desiredAngle);
	

	void End_Action (bool drive, bool turn);

	// checks if we are off course
	bool checkCourse();
	
	typedef enum {delay, delay_check, drive_forward, drive_forward_check, twist, twist_check, 
		drive_tube_peg, drive_tube_peg_check, ejection, ejection_check, reverse,
		reverse_check, done, error} Straight_States; 
	
	typedef enum {fork_delay, fork_delay_check, drive_forward_fork, drive_forward_fork_check, second_drive, second_drive_check,
		/*turn_fork, turn_fork_check,*/ ejection_prep, ejection_prep_check, 
		/*thrid_drive, third_drive_check,*/ little_turn, little_turn_check, drive_peg, 
		drive_peg_check, fork_ejection, fork_ejection_check,
		drive_fork_back, drive_back_fork_check, turn, 
		turn_check, last_fork_drive, last_fork_fork_drive_check, fork_done, fork_error} Fork_States;
	
	typedef enum {dead_delay, dead_delay_check, dead_drive_forward, dead_drive_forward_check, dead_twist, dead_twist_check, 
		dead_drive_tube_peg, dead_drive_tube_peg_check, dead_ejection, dead_ejection_check, dead_reverse,
		dead_reverse_check, dead_done, undead_error} Dead_States; 
	
	typedef enum {raise_arm_start, raise_arm_stop, drive_back_start, drive_back_stop,
		turn_start, turn_stop, finish, ended, end_error} End_Subphase;



	Straight_States straight_state; 
	Fork_States fork_state;
	Dead_States dead_state;
	End_Subphase subphase;

	ThunderDrive * ThunderDriveCode;
	ThunderArm * ThunderArmCode; 
	ThunderManip * ThunderManipCode;
	UDPLogger * logger;

	//to make sure we are still driving on course.
	float minAngle;					// min angle we need to stay within
	float maxAngle;					// max angle we need to stay within

};

#endif
