#include "Resources.h"
#include "ThunderArm.h"
#include "ThunderDrive.h"
#include "ThunderManip.h"
#include "ThunderControls.h"
#include "ThunderDeployer.h"
#include "ThunderLight.h"
#include "ThunderAuto.h"

//#define TESTING_TRACKING
//#define TESTING_TURNING

class ThunderBot : public SimpleRobot
{
	ThunderControls * thunderControls;
	ThunderArm * thunderArm;
	ThunderManip * thunderManip;
	ThunderDrive * thunderDrive;
	ThunderDeployer * thunderDeployer;
	ThunderLight * thunderLight;
	ThunderAuto * thunderAuto;
	double lightsGoNormalTime;
	
	
public:
	ThunderBot(void)
	{
		thunderDeployer = new ThunderDeployer(false);
		thunderArm = new ThunderArm(thunderDeployer);
		thunderLight = new ThunderLight();
		thunderLight->DoPrematchSignaling(ThunderLight::PREMATCH_GYROCALIB);
		thunderDrive = new ThunderDrive();
		thunderLight->DoPrematchSignaling(ThunderLight::PREMATCH_GYRODONE);
		lightsGoNormalTime = GetTime() + 10;
		thunderManip = new ThunderManip();
		thunderControls = new ThunderControls(thunderDrive, thunderArm, thunderManip, thunderDeployer, thunderLight, thunderAuto);
		thunderAuto = new ThunderAuto(thunderDrive, thunderArm, thunderManip);
	}
	~ThunderBot(void)
	{
		delete thunderAuto;
		delete thunderControls;
		delete thunderManip;
		delete thunderDrive;
		delete thunderLight;
		delete thunderArm;
		delete thunderDeployer;
	}

	void trackingTest()
	{
#ifdef TESTING_TRACKING		// set up line tracking
		thunderDrive->Set_Coast(false);
		thunderDrive->Drive_Keep_On_Line(150);
		while(!IsOperatorControl()) {
			// keep tracking line and check if we've reached the end
			thunderDrive->Drive_Process_Auto();
			if(thunderDrive->Drive_Current_Auto_Task_Done())
				break;
			Wait(0.005);
		}
		printf("We're done with straight-line! Oh no!\n");
		thunderDrive->Drive_Forward_Distance(16);
		while(!IsOperatorControl()) {
			// keep tracking line and check if we've reached the end
			thunderDrive->Drive_Process_Auto();
			if(thunderDrive->Drive_Current_Auto_Task_Done())
				break;
			Wait(0.005);
		}
#endif
#ifdef TESTING_TURNING
		// set up line tracking
		thunderDrive->Set_Coast(false);
		thunderDrive->Drive_Turn_Until_Line(-35.0);
		while(!IsOperatorControl()) {
			// keep turning and check if we've reached the line
			thunderDrive->Drive_Process_Auto();
			if(thunderDrive->Drive_Current_Auto_Task_Done())
				break;
			Wait(0.005);
		}
		thunderDrive->Drive_Keep_On_Line(36, false);
		thunderDrive->Drive_Set_Max_Auto_Speed(0.2); // force slower speed
		while(!IsOperatorControl()) {
					// keep turning and check if we've reached the line
			thunderDrive->Drive_Process_Auto();
			if(thunderDrive->Drive_Current_Auto_Task_Done())
				break;
			Wait(0.005);
		}
		printf("We're done! Oh no!\n");
#endif	
	}

	void Autonomous(void)
	{
		ThunderAuto::Approach approach;
		ThunderAuto::Peg_Height pegHeight;
		ThunderAuto::Ending_Action endAction;
		float autoDelay;
		
		thunderControls->StartTimer();

		//Reset functions
		thunderLight->DoPrematchSignaling(ThunderLight::PREMATCH_COMPLETE);
		thunderLight->SetColors(false, false, false, false);
		thunderArm->ArmResetAfterAuto();
		thunderManip->ManipResetAfterAuto();
		thunderDrive->Drive_Stop_Auto_Task();
		
		thunderControls->RelayAutoValues((int*)&approach, (int*)&pegHeight, (int*)&endAction, &autoDelay);
		printf("autoApproach:%i autoPegHeight:%i autoEndAction:%i\n", (int)approach, (int)pegHeight, (int)endAction);
		printf("Auto delay:%f\n", autoDelay);
		thunderDrive->Set_Coast(false);
		thunderAuto->Initialize(autoDelay); //autoDelay);
		thunderDrive->Drive_Set_Max_Auto_Speed(0.6);
		while(IsAutonomous() && IsEnabled())
		{
			thunderControls->ReadBrokenSwitches();
			thunderControls->ProcessLightshow();
			thunderControls->RunDashboard();
			thunderDrive->Drive_Process_Auto();
			thunderArm->ArmProcess();
			thunderManip->ManipProcessAuto();
			thunderManip->ManipProcess();

			if	(!thunderArm->IsArmBroken())
			{
				thunderAuto->run_auto(approach, pegHeight, endAction);
			}
			Wait(0.005);
		}
	}
	
	void Disabled(void)
	{
		bool lightsSetNormal = false;
		thunderDeployer->Reset();
		thunderDrive->Set_Coast(true);
		thunderControls->StopTimer();
		while(IsDisabled())
		{
			thunderControls->RunDebug();
			if (lightsSetNormal || GetTime() > lightsGoNormalTime) {
				if (!lightsSetNormal) {
					lightsSetNormal = true;
					thunderLight->DoPrematchSignaling(ThunderLight::PREMATCH_COMPLETE);
				}
				thunderControls->ProcessLightshow();
			}
			thunderControls->RunDashboard();
			Wait(0.005);
		}
	}

	void OperatorControl(void)
	{
		//Reset functions
		thunderArm->ArmResetAfterAuto();
		thunderManip->ManipResetAfterAuto();
		thunderDrive->Drive_Stop_Auto_Task();
		thunderLight->DoPrematchSignaling(ThunderLight::PREMATCH_COMPLETE);
		thunderLight->SetColors(false, false, false, false);
		thunderControls->Reset();
		
		thunderDeployer->StartMatch();
		thunderDrive->Set_Coast(false);
		while (IsOperatorControl() && IsEnabled())
		{
			thunderControls->ReadBrokenSwitches();
			thunderControls->RunControls();
			thunderControls->RunDebug();
			thunderControls->ProcessCamera();
			thunderControls->RunDashboard();
			thunderArm->ArmProcess();
			thunderManip->ManipProcess();
			thunderDeployer->ProcessDeployer();
			Wait(0.005);				// wait for a motor update time
		}
	}
};

START_ROBOT_CLASS(ThunderBot);

