#include "Resources.h"
#include "DSmap.h"
#include "ThunderControls.h"
#include <stdarg.h>

ThunderControls::ThunderControls(ThunderDrive* Drive, ThunderArm* Arm, 
			ThunderManip* Manip, ThunderDeployer* Deployer, 
			ThunderLight* Light, ThunderAuto* Auto):
	PrimaryLeft(DS_JOY_LEFT_PORT),
	PrimaryRight(DS_JOY_RIGHT_PORT),
	SecondaryJoy(DS_JOY_SECONDARY_PORT),
	ExtendedIOBoard(DS_JOY_IOBOARD_PORT),
	CameraServo(SERV_CAM_TILT),
	driverStation(DriverStation::GetInstance()),
	driverStationLCD(DriverStationLCD::GetInstance())
{
	//Object and pointer constructors
	thunderDrive = Drive;
	thunderArm = Arm;
	thunderManip = Manip;
	thunderDeployer = Deployer;
	thunderLight = Light;
	thunderAuto = Auto;
	lastButtonState = false;
	lightshowNumber = 0;
	timeLeft = new Timer();
	lastPreset = ThunderArm::PRE_NONE;
	presetFlipped = false;
}

ThunderControls::~ThunderControls()
{
	delete timeLeft;
}

void ThunderControls::Reset()
{
	lastPreset = ThunderArm::PRE_NONE;
	presetFlipped = false;
}

void ThunderControls::RunControls()
{
	RunControlSticks();
	RunControlBoard();
	/*printf(
			"%c:%c:%c:%c:%c:%c:%c:%c:%c:%c:%c:%c:%f:%f:%f:%f:%f:%f\n",
		SecondaryJoy.GetRawButton(1), SecondaryJoy.GetRawButton(2),
		SecondaryJoy.GetRawButton(3), SecondaryJoy.GetRawButton(4),
		SecondaryJoy.GetRawButton(5), SecondaryJoy.GetRawButton(6),
		SecondaryJoy.GetRawButton(7), SecondaryJoy.GetRawButton(8),
		SecondaryJoy.GetRawButton(9), SecondaryJoy.GetRawButton(10),
		SecondaryJoy.GetRawButton(11), SecondaryJoy.GetRawButton(12),
		(double)SecondaryJoy.GetRawAxis(1), (double)SecondaryJoy.GetRawAxis(2),
		(double)SecondaryJoy.GetRawAxis(3), (double)SecondaryJoy.GetRawAxis(5),
		(double)SecondaryJoy.GetRawAxis(4), (double)SecondaryJoy.GetRawAxis(6)
	);*/
}

void ThunderControls::RunControlSticks()
{
	bool driveBackwards;
	if(PrimaryLeft.GetRawButton(DS_JOYBTN_REVERSE) || PrimaryRight.GetRawButton(DS_JOYBTN_REVERSE)
			|| PrimaryLeft.GetRawButton(DS_JOYBTN_REVERSE_ALT) || PrimaryRight.GetRawButton(DS_JOYBTN_REVERSE_ALT))
	{
		driveBackwards = true;
	}
	else
	{
		driveBackwards = false;
	}
	//Drive the motors using scaled joystick values
	Joystick * switchJoy[2];
	if(driveBackwards)
	{
		switchJoy[1] = &PrimaryLeft;
		switchJoy[0] = &PrimaryRight;
	}
	else
	{
		switchJoy[0] = &PrimaryLeft;
		switchJoy[1] = &PrimaryRight;
	}
	thunderDrive->Drive_Left_Right_Motors(GetPower(switchJoy[0], driveBackwards),
		GetPower(switchJoy[1], driveBackwards));
	
	//Arm control
	//printf("Joystick: %f\n)", SecondaryJoy.GetY());
	thunderArm->ArmGoUpDown(0.75 * (SecondaryJoy.GetY() * SecondaryJoy.GetY() * SecondaryJoy.GetY()* SecondaryJoy.GetY() * SecondaryJoy.GetY()));
	if(SecondaryJoy.GetY() <= .05 && SecondaryJoy.GetY() >= -.05)
	{
		thunderArm->ArmSetAdhocPreset();
	}
	else
	{
		thunderArm->ArmStopAdhoc();
	}
	//Preset control here
	//thunderArm->ArmProcess();
	
	//Manip control
	thunderManip->ManipTwist(SecondaryJoy.GetRawAxis(6)*-0.5);

	thunderManip->ManipInOut(GetInOut(&SecondaryJoy));
	
}

void ThunderControls::RunControlBoard()
{
	//Arm presets
	ThunderArm::PresetPositions preset;
	
	if (SecondaryJoy.GetRawButton(DS_JOYBTN_PRESET_STRAIGHT))
	{
		preset = ThunderArm::PRE_STRAIGHT_UP;
	}
	else if (SecondaryJoy.GetRawButton(DS_JOYBTN_AUTO_OFFSET_PRESET))
	{
		// move arm to autonomous offset peg heights
		if (SecondaryJoy.GetRawButton(DS_JOYBTN_AUTO_LOWER_PRESET))
		{
			// use the "lower" positions
			if(ExtendedIOBoard.GetRawButton(DS_PRESET_HIGH))
			{
				preset = ThunderArm::PRE_AUTO_HIGH_OFFSET_LOWER;
			}
			else if(ExtendedIOBoard.GetRawButton(DS_PRESET_MID))
			{
				preset = ThunderArm::PRE_AUTO_MED_OFFSET_LOWER;
			}
			else if(ExtendedIOBoard.GetRawButton(DS_PRESET_LOW))
			{
				preset = ThunderArm::PRE_AUTO_LOW_OFFSET_LOWER;
			}
			else
			{
				preset = ThunderArm::PRE_NONE;
			}
		}
		else
		{
			// use the "placement" positions
			if(ExtendedIOBoard.GetRawButton(DS_PRESET_HIGH))
			{
				preset = ThunderArm::PRE_AUTO_HIGH_OFFSET;
			}
			else if(ExtendedIOBoard.GetRawButton(DS_PRESET_MID))
			{
				preset = ThunderArm::PRE_AUTO_MED_OFFSET;
			}
			else if(ExtendedIOBoard.GetRawButton(DS_PRESET_LOW))
			{
				preset = ThunderArm::PRE_AUTO_LOW_OFFSET;
			}
			else
			{
				preset = ThunderArm::PRE_NONE;
			}
		}
	}
	else if (SecondaryJoy.GetRawButton(DS_JOYBTN_AUTO_PRESET))
	{
		// move arm to autonomous normal peg heights
		if (SecondaryJoy.GetRawButton(DS_JOYBTN_AUTO_LOWER_PRESET))
		{
			// use the "lower" positions
			if(ExtendedIOBoard.GetRawButton(DS_PRESET_HIGH))
			{
				preset = ThunderArm::PRE_AUTO_HIGH_LOWER;
			}
			else if(ExtendedIOBoard.GetRawButton(DS_PRESET_MID))
			{
				preset = ThunderArm::PRE_AUTO_MED_LOWER;
			}
			else if(ExtendedIOBoard.GetRawButton(DS_PRESET_LOW))
			{
				preset = ThunderArm::PRE_AUTO_LOW_LOWER;
			}
			else
			{
				preset = ThunderArm::PRE_NONE;
			}
		}
		else
		{
			// use the "placement" positions
			if(ExtendedIOBoard.GetRawButton(DS_PRESET_HIGH))
			{
				preset = ThunderArm::PRE_AUTO_HIGH;
			}
			else if(ExtendedIOBoard.GetRawButton(DS_PRESET_MID))
			{
				preset = ThunderArm::PRE_AUTO_MED;
			}
			else if(ExtendedIOBoard.GetRawButton(DS_PRESET_LOW))
			{
				preset = ThunderArm::PRE_AUTO_LOW;
			}
			else
			{
				preset = ThunderArm::PRE_NONE;
			}
		}
	}
	else if(ExtendedIOBoard.GetRawButton(DS_PRESET_HIGH))
	{
		preset = ThunderArm::PRE_HIGH;
	}
	else if(ExtendedIOBoard.GetRawButton(DS_PRESET_MID))
	{
		preset = ThunderArm::PRE_MEDIUM;
	}
	else if(ExtendedIOBoard.GetRawButton(DS_PRESET_LOW))
	{
		preset = ThunderArm::PRE_LOW;
	}
	else if(ExtendedIOBoard.GetRawButton(DS_PRESET_FEED))
	{
		preset = ThunderArm::PRE_FEEDER;
	}
	else if(ExtendedIOBoard.GetRawButton(DS_PRESET_GROUND))
	{
		preset = ThunderArm::PRE_FLOOR;
	}	
	else
	{
		preset = ThunderArm::PRE_NONE;
	}
	
	if (preset != lastPreset) {
		presetFlipped = SecondaryJoy.GetRawButton(DS_JOYBTN_ARM_PRESET_FLIP);
		lastPreset = preset;
	}
	thunderArm->ArmGotoPreset(preset, presetFlipped);
	
	//thunderDeployer->OverrideTimer(true);
	
	//Tusks and minibot deploy code
	if((ExtendedIOBoard.GetRawButton(DS_BUTTON_TUSKS_DEPLOY) || SecondaryJoy.GetRawButton(DS_JOYBTN_TUSKS_DEPLOY)) 
			&& thunderArm->ArmCheckTusksReady())
	{
		thunderDeployer->DeployTusks();
	}
	if((ExtendedIOBoard.GetRawButton(DS_BUTTON_MINIBOT_DEPLOY) || SecondaryJoy.GetRawButton(DS_JOYBTN_MINIBOT_DEPLOY)) 
			&& thunderArm->ArmCheckTusksReady() 
			&& thunderArm->ArmCheckMinibotReady())
	{
		thunderDeployer->DeployMinibot();
	}
	else if ((ExtendedIOBoard.GetRawButton(DS_BUTTON_TUSKS_DEPLOY) || SecondaryJoy.GetRawButton(DS_JOYBTN_TUSKS_DEPLOY)) && thunderDeployer->GetTusksDeployed())
	{
		thunderDeployer->UndeployMinibot();
	}
	//thunderDeployer->ProcessDeployer();
	
	//Controls the light tower	
	bool red = ExtendedIOBoard.GetRawButton(DS_BUTTON_TRIANGLE);
	bool white = ExtendedIOBoard.GetRawButton(DS_BUTTON_CIRCLE);
	bool blue = ExtendedIOBoard.GetRawButton(DS_BUTTON_SQUARE);
	
	//override color if we have a tube
	if(thunderManip->ManipLight())
	{
		//Turn off all request lights and turn on green light
		thunderLight->SetColors(false, false, false, true);
	}
	else
	{
		//turn off green light, use the inputs from IO board
		thunderLight->SetColors(red, white, blue, false);
	}
}


void ThunderControls::RunDebug()
{
	//finds if a debug button is pushed and sets debugButton with the value
	int debugButton = 0;
	for(int i = 6; i <= 11; i++)
	{
		if(PrimaryLeft.GetRawButton(i))
		{
			debugButton = i;
		}
	}
	
	//THE BEHEMOTH THAT IS DEBUG
	switch(debugButton){
	case 0:
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line1, "");
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line2, "");
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line3, "");
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line4, "");
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line5, "");
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line6, "");
		driverStationLCD->UpdateLCD();
		break;
	case DS_JOYBTN_DEBUG_DRIVETRAIN:
		printf("encoderL:%i encoderR:%i gyro:%f\n", 
				thunderDrive->Get_Encoder_Ticks(true), 
				thunderDrive->Get_Encoder_Ticks(false),
				(double)thunderDrive->Get_Gyro_Angle());
//		printf("Faultstate: %d %f\n", thunderDrive->motor[2]->GetFaults() & 0xffff, (double)thunderDrive->motor[2]->GetTemperature());
		
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line1, 
			"gyro angle: %f", (double)thunderDrive->Get_Gyro_Angle());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line2, 
			"encoderL: %i", thunderDrive->Get_Encoder_Ticks(true));
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line3, 
			"encoderR: %i", thunderDrive->Get_Encoder_Ticks(false));
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line4,"");
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line5,"");
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line6,"");
		driverStationLCD->UpdateLCD();
		break;
	case DS_JOYBTN_DEBUG_LINE:
		printf("sensor1:%i sensor2:%i sensor3:%i\n", 
				(int)thunderDrive->Sensor_Active(0), 
				(int)thunderDrive->Sensor_Active(1), 
				(int)thunderDrive->Sensor_Active(2));
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line1, 
			"Sensor 1: %i", (int)thunderDrive->Sensor_Active(0));
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line2, 
			"Sensor 2: %i", (int)thunderDrive->Sensor_Active(1));
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line3, 
			"Sensor 3: %i", (int)thunderDrive->Sensor_Active(2));
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line4, "");
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line5, "");
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line6, "");
		driverStationLCD->UpdateLCD();
		break;
	case DS_JOYBTN_DEBUG_ARM:
		printf("armAngle:%f Limits F:%i B:%i Calibrate:%i Manip Banner:%f\n", 
				(double)thunderArm->ArmDebugAngle(),
				(int)thunderArm->ArmDebugFrontLimit(),
				(int)thunderArm->ArmDebugBackLimit(),
				(int)thunderArm->ArmDebugCalibLimit(),
				(double)thunderManip->ManipGetBannerSensor());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line1, 
			"Arm Angle: %f", (double)thunderArm->ArmDebugAngle());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line2, 
			"Limits F:%i B:%i", (int)thunderArm->ArmDebugFrontLimit(),
								(int)thunderArm->ArmDebugBackLimit());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line3, 
			"Limit Calibrate: %i", (int)thunderArm->ArmDebugCalibLimit());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line4, 
			"Manip Banner: %f", (double)thunderManip->ManipGetBannerSensor());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line5, 
			"Left Encoder:%i", thunderArm->ArmDebugEncoderLeft());
		driverStationLCD->PrintfLine(DriverStationLCD::kUser_Line6, 
			"Right Encoder:%i", thunderArm->ArmDebugEncoderRight());
		driverStationLCD->UpdateLCD();
	default:
		break;
	}
}

void ThunderControls::RelayAutoValues(int* startPosReturn, int* pegHeightReturn, int* endPosReturn, float* autoDelayReturn)
{
	//first 4 bits of analog are startPos
	//second 4 bits of analog are endPos
	//digital inputs are pegHeight
	int startPos = 0;
	int pegHeight = 0;
	int endPos = 0;
	bool digital1 = ExtendedIOBoard.GetRawButton(DS_IO_AUTOMODE_DIGITAL_1);
	bool digital2 = ExtendedIOBoard.GetRawButton(DS_IO_AUTOMODE_DIGITAL_2);
	INT8 rawAutoValTop = driverStation->GetStickAxisRaw(DS_JOY_IOBOARD_PORT, DS_IO_ANALOG_AUTOMODE_CHANNEL_TOP) + 1;
	INT8 rawAutoValBottom = driverStation->GetStickAxisRaw(DS_JOY_IOBOARD_PORT, DS_IO_ANALOG_AUTOMODE_CHANNEL_BOTTOM) + 1;
	INT8 rawAutoDelay = driverStation->GetStickAxisRaw(DS_JOY_IOBOARD_PORT, DS_IO_ANALOG_AUTODELAY_CHANNEL) + 1;
	
	
	rawAutoValTop = rawAutoValTop & 60; //0x3C
	rawAutoValBottom = rawAutoValBottom & 60;
	rawAutoDelay = (rawAutoDelay >> 1) & 15;
	printf("RAW AUTO DELAY:%x Dec:%i\n", rawAutoDelay, rawAutoDelay);
	INT8 rawAutoVal = (rawAutoValTop << 2) | (rawAutoValBottom >> 2);
	
	for(int i = 0; i < 4; i++)
	{
		startPos += rawAutoVal & (1 << i);
	}
	for(int i = 4; i < 8; i++)
	{
		endPos += rawAutoVal & (1 << i);
	}
	endPos >>= 4;
	pegHeight = (int)digital1 + 2*(int)digital2;
	
	*startPosReturn = startPos;
	*pegHeightReturn = pegHeight;
	*endPosReturn = endPos;
	*autoDelayReturn = (float)rawAutoDelay;
	//thunderAuto->run_auto((ThunderAuto::Approach)startPos, (ThunderAuto::Peg_Height)pegHeight, (ThunderAuto::Ending_Action)endPos);
	
}

void ThunderControls::ReadBrokenSwitches()
{
	bool switchStates[8];
	INT8 rawAnalogValueTop = driverStation->GetStickAxisRaw(DS_JOY_IOBOARD_PORT, DS_IO_ANALOG_SWITCHBANK_CHANNEL_TOP) + 1;
	INT8 rawAnalogValueBottom = driverStation->GetStickAxisRaw(DS_JOY_IOBOARD_PORT, DS_IO_ANALOG_SWITCHBANK_CHANNEL_BOTTOM) + 1;
	rawAnalogValueTop = rawAnalogValueTop & 60;  //0x3C
	rawAnalogValueBottom = rawAnalogValueBottom & 60;
	INT8 rawAnalogValue = (rawAnalogValueTop << 2) | (rawAnalogValueBottom >> 2);
	//printf("%x", rawAnalogValue);
	for(int i = 0; i < 8; i++)
	{
		switchStates[i] = rawAnalogValue & (1 << i);
	}
	
	if(switchStates[DS_BROKEN_MANIP]) 	
		thunderManip->ManipBannerSensorBroken(true);		
	else 
		thunderManip->ManipBannerSensorBroken(false);
	if(switchStates[DS_BROKEN_ARM_FORWARD]) 
		thunderArm->ArmFrontLimitBroken(true);
	else 
		thunderArm->ArmFrontLimitBroken(false);
	if(switchStates[DS_BROKEN_ARM_BACKWARD]) 
		thunderArm->ArmBackLimitBroken(true);
	else 
		thunderArm->ArmBackLimitBroken(false);
	if(switchStates[DS_BROKEN_ARM_CALIBRATE]) 
		thunderArm->ArmCalibLimitBroken(true);
	else 
		thunderArm->ArmCalibLimitBroken(false);
	if(switchStates[DS_ARM_ENCODER_SELECT_A])
	{
		thunderArm->ArmEncoderSwap(false);
		thunderArm->ArmEncoderBroken(false);
	}
	else if(switchStates[DS_ARM_ENCODER_SELECT_B])
	{
		thunderArm->ArmEncoderSwap(true);
		thunderArm->ArmEncoderBroken(false);
	}
	else 
		thunderArm->ArmEncoderBroken(true);
	if(switchStates[DS_DEPLOYER_TIMER_OVERRIDE])
		thunderDeployer->OverrideTimer(true);
	else
		thunderDeployer->OverrideTimer(false);
	if(switchStates[DS_ARM_ADHOC_DISABLE])
		thunderArm->ArmDisableAdhoc(true);
	else
		thunderArm->ArmDisableAdhoc(false);
}

void ThunderControls::ProcessLightshow()
{
	bool buttonState = PrimaryLeft.GetRawButton(4);
	if(buttonState && (buttonState != lastButtonState))
	{
		lightshowNumber++;
		if(lightshowNumber > 12)  //replace with the number of lighshows - 1
			lightshowNumber = 0;
		thunderLight->ClearLights();
	}
	lastButtonState = buttonState;
	
	switch(lightshowNumber)
	{
	case 0:
		thunderLight->WarningFlash();
		break;
	case 1:
	    thunderLight->SineFlash();
	    break;
	case 2:
		thunderLight->FadeUpDown();
		break;
	case 3:
		thunderLight->OnOffFlash();
		break;
	case 4:
		thunderLight->LightChaseDown();
		break;
	case 5:
		thunderLight->LightChaseUp();
		break;
	case 6:
		thunderLight->LightChaseUpDown();
		break;
	case 7:
		thunderLight->LightRandom();
		break;
	case 8:
		thunderLight->FadeInOut();
		break;
	case 9:
		thunderLight->FireLaser();
		break;
	case 10:
		thunderLight->FireLaser2();
		break;
	case 11:
		thunderLight->Binary();
		break;
    case 12:
        thunderLight->SinePulse();
        break;
	default:
		break;
	}
	
	
	
}

void ThunderControls::RunDashboard()
{
	Dashboard &dash = DriverStation::GetInstance()->GetLowPriorityDashboardPacker();
	dash.AddCluster();
		{
			dash.AddCluster();
			{ //analog modules 
				dash.AddCluster();
				{
					for (int i = 1; i <= 8; i++) {
						dash.AddFloat((float) AnalogModule::GetInstance(1)->GetAverageVoltage(i));
					}
				}
				dash.FinalizeCluster();
			}
			dash.FinalizeCluster();
				
			dash.AddCluster();
			{ //digital modules
				dash.AddCluster();
				{
					int module = 4;
					dash.AddU8(DigitalModule::GetInstance(module)->GetRelayForward());
					dash.AddU8(DigitalModule::GetInstance(module)->GetRelayReverse());
					dash.AddU16((short)DigitalModule::GetInstance(module)->GetDIO());
					//dash.AddU16((short) 0xAAAA);
					dash.AddU16((short)DigitalModule::GetInstance(module)->GetDIODirection());
					//dash.AddU16((short) 0x7777);
					dash.AddCluster();
					{
						for (int i = 1; i <= 10; i++) {
							dash.AddU8((unsigned char) DigitalModule::GetInstance(module)->GetPWM(i));
							//dash.AddU8((unsigned char) (i-1) * 255 / 9);
							}
					}
					dash.FinalizeCluster();
				}
				dash.FinalizeCluster();
			}
			dash.FinalizeCluster();
			dash.AddU8((char) 0);//solenoids
			unsigned char boolCompress = 
				(
					(char)
					(
						((int)thunderManip   ->ManipGetBannerSensor()   )+
						((int)thunderArm     ->ArmCheckPresetDone  ()<<1)+
						((int)thunderArm     ->GetArmInverted      ()<<2)+
						((int)thunderDeployer->MinibotReady        ()<<3)+
						((int)thunderArm     ->ArmDebugBackLimit   ()<<4)+
						((int)thunderArm     ->ArmDebugFrontLimit  ()<<5)
					)
				);
			//Add custom data here
			dashsendf
			(
					dash, 
					"ci\0",
					boolCompress, thunderArm->ArmDebugAngle()/4+24
			);
			dash.AddDouble((double)(int)timeLeft->Get());
			
		}
		dash.FinalizeCluster();

	dash.Finalize();
}

void ThunderControls::StartTimer()
{
	timeLeft->Reset();
	timeLeft->Start();
}

void ThunderControls::StopTimer()
{
	timeLeft->Stop();
}

void ThunderControls::ProcessCamera()
{
	int cameraAngle = thunderArm->ArmDebugAngle()/4 - 45;
	if(cameraAngle > 90)
	{
		cameraAngle = 90;
	}
	CameraServo.SetAngle(180 - (float)cameraAngle);
}

//Pegasus

float ThunderControls::GetPower(Joystick* joystick, bool backwards)
{
	if(joystick->GetY() > .1 || joystick->GetY() < -.1)//deadzone
	{
		//Cube the power for a nice curve
		float power = joystick->GetY() * joystick->GetY() * joystick->GetY();
		if (joystick->GetRawButton(DS_JOYBTN_TURBO))
		{
			//Keep full power
		}
		else if(joystick->GetRawButton(DS_JOYBTN_SLOW))
		{
			power *= .3; // 3/10 power
		}
		else
		{
			power *= .6;  // 5/10 power (was .7 and was too much - JDK @ FLR)
		}
		if(backwards)
		{
			power *= -1;  //reverse the direction
		}
		return -power;
	}
return 0;
}

float ThunderControls::GetInOut(Joystick* joystick)
{
	float dir = 0;
	if(joystick->GetRawButton(DS_JOYBTN_MANIP_IN) && joystick->GetRawButton(DS_JOYBTN_MANIP_OUT))
	{
		//keep dir = 0
	}
	else if(joystick->GetRawButton(DS_JOYBTN_MANIP_IN))
	{
		dir = -1;  //absorb tube
		//add multiplier for the control knob
	}
	else if(joystick->GetRawButton(DS_JOYBTN_MANIP_OUT))
	{
		dir = 1;  //POOSH LITTLE TUBE
		//add multiplier for the control knob
	}
	return dir;
}


void ThunderControls::dashsendf(Dashboard &dash, char * format, ...)
{
	va_list ap;
	va_start(ap, format);
	int a = 0;
	while(1)
	{
		if(format[a] == '\0') break;
		a++;
	}
	//dash.AddCluster();
	for(int b = 0; b < a; b++)
	{
		switch(format[b])
		{
		case 'c':
			dash.AddI8((char)va_arg(ap, int));
			break;
		case 's':
			dash.AddI16((short)va_arg(ap, int));
			break;
		case 'i':
			dash.AddI32(va_arg(ap, int));
			break;
		case 'f':
			dash.AddFloat((float)va_arg(ap, double));
			break;
		case 'd':
			dash.AddDouble(va_arg(ap, double));
			break;
		}
	}
	//dash.FinalizeCluster();
}
