#ifndef THUNDER_CONTROLS_H
#define THUNDER_CONTROLS_H

#include "Resources.h"
#include "ThunderDrive.h"
#include "ThunderArm.h"
#include "ThunderManip.h"
#include "ThunderDeployer.h"
#include "ThunderLight.h"
#include "ThunderAuto.h"

/*
 user input, driver station functions
 
 TASKS TO BE DONE:
 	-Autonomus initilization (test)
 	-Broken switches (test)
 	-Tusks/Deployer (test)


*/

class ThunderControls
{
public:
	ThunderControls(ThunderDrive* Drive, ThunderArm* Arm, ThunderManip* Manip, ThunderDeployer* Deployer, ThunderLight* Light, ThunderAuto* Auto);
	~ThunderControls();
	
	//Runs the teleop controls code. call every loop
	void RunControls();
	
	//Gets the automode numbers from the DS and sends them to ThunderAuto
	void RelayAutoValues(int* startPosReturn, int* pegHeightReturn, int* endPosReturn, float* autoDelay);
	
	//Reads the broken switches
	void ReadBrokenSwitches();
	
	//Checks the joysticks for the debug buttons and prints to LCD
	void RunDebug();
	
	//Run this to be able to control which lightshow to play in disabled
	void ProcessLightshow();
	
	//Run this to pack and send the dashboard data
	void RunDashboard();
	
	void dashsendf(Dashboard &dash, char * format, ...);
	
	//Start the timeLeft timer
	void StartTimer();
	
	void StopTimer();
	
	//Run this to move the camera to the correct angle
	void ProcessCamera();
	
	// Reset any internal state; call when robot has not been
	// looking at controls (RunControls()) for a period of time
	// such that the physical controls states may have changed without
	// the this class having had a chance to read them.
	// For example, after having been disabled/in autonomous for a time
	void Reset();
	
private:
	//Joystick objects
	Joystick PrimaryLeft;
	Joystick PrimaryRight;
	Joystick SecondaryJoy;
	Joystick ExtendedIOBoard;
	
	Servo CameraServo;
	
	//DriverStation components
    DriverStation *driverStation;
    DriverStationLCD * driverStationLCD;
    
	//ThunderComponents pointers
	ThunderDrive* thunderDrive;
	ThunderArm* thunderArm;
	ThunderManip* thunderManip;
	ThunderDeployer* thunderDeployer;
	ThunderLight* thunderLight;
	ThunderAuto* thunderAuto;
	
	Timer* timeLeft;
	
	bool lastButtonState;
	int lightshowNumber;
	
	ThunderArm::PresetPositions lastPreset;
	bool presetFlipped;

	//processes input from operator joysticks.
	void RunControlSticks();
	
	//processes IO from DS buttons/knobs/doohickeys
	void RunControlBoard();
	
	
	//gets the power of a joystick depending on turbo/slow buttons
	//Arguments: 
		//pointer to a joystick to read,
		//Backwards: inverts the motor direction
	float GetPower(Joystick* joystick, bool backwards);
	
	//gets the correct value for the ThunderManip->ManipInOut() function from 
	// the buttons on the throttle
	//Arguments: Pointer of a joystick to read from
	float GetInOut(Joystick* joystick);
};

#endif
