#include "ThunderDrive.h"
#include "Resources.h"
#include "ThunderDrive.h"
#include "Tracker.h"
#include <math.h>

#define MIN_DRIVE_STOP_DISTANCE 20
#define MIN_TURN_STOP_ANGLE 15
#define fabs(a) ((a)<0?-(a):(a))
#define GET_SIGN(a) (a<0?-1:(a>0?1:0))
#define CIM_RPM 4614
#define CIM_RPS ((CIM_RPM)/60)
#define WHEEL_CIRCUMFERENCE 25.132
#define TICKS_PER_REVOLUTION 900
#define INCHES_PER_TICK 0.027924
#define CLAMP_VALUE(v, b, g) v = ((g)?((v>b)?(b):(v)):((v<b)?(b):(v)))
#define GET_LARGER(a, b) (a>b)?(a):(b)
#define GET_LESSER(a, b) (a<b)?(a):(b)
#define DEAD_RECKONNING_DEBUG 0

// minimum value that will run the motors
#define MIN_SPEED 0.15
// speed to turn at to find the line
#define TURN_SPEED 0.22

//					Line tracking constants:
// number of times that robot can be off line before we give up
#define MISS_TRIES 10
// number of attempts to detect line at start of autonomous before we give up
#define START_TRIES 10
// number of times we must see an end value consecutively to believe we've
// reached the end...
#define END_TRIES 3
/* Maximum speed necessary to find stop bar three times consecutively
// (maximum speed where we can quickly and definitively know we're done)
// Provisional but semi-tested; BUT ARE WE USING IT? apparently not...
#define MAX_END_SPEED 0.3*/
// factor by which to decrease speed of left side to turn left
// or right side to turn right
#define TURN_FACTOR 0.5
// same as above, but when only slightly off line, so needing less
// strong correction
#define TURN_FACTOR_SLIGHT sqrt(TURN_FACTOR)
// minimum distance over which we should be allowed to accelerate (inches)
#define ACCEL_DISTANCE 60
// distance we will drive at end at MIN_SPEED (inches)
#define END_DISTANCE 40


//0: no correction
//>0: correct right
//<0: correct left
#define DIRECTION_CORRECTION 0

#define DIGITAL_GYRO 1
#define DIGITAL_OPTICAL_SENSOR(index) 1+index


ThunderDrive::ThunderDrive()
{
	l_speed            = 0   ;
	r_speed            = 0   ;
	current_auto_task  = none;
	auto_start         = 0   ;
	auto_progress      = 0   ;
	auto_goal          = 0   ;
	ramp_state         = 0   ;
	max_auto_speed     = 0.35;
	current_auto_speed = 0;
	
	motor[0]     = new ThunderCANJaguar(CANID_DRIVETRAIN_LEFT   );
	motor[1]     = new ThunderCANJaguar(CANID_DRIVETRAIN_RIGHT  );
#if NUM_CIMS > 2
	motor[2]     = new ThunderCANJaguar(CANID_DRIVETRAIN_LEFT_2 );
	motor[3]     = new ThunderCANJaguar(CANID_DRIVETRAIN_RIGHT_2);
#endif
	encoder[0]   = new Counter  (DIG_ENCODER_LEFT_A    );
	encoder[1]   = new Counter  (DIG_ENCODER_RIGHT_A   );
	speed_timer  = new Timer    (                      );
	tracker      = new Tracker  (ANA_LINE_LEFT, 
								ANA_LINE_CENTER,
								ANA_LINE_RIGHT		   );
	thunder_gyro = new Gyro     (1                     );
	thunder_gyro->SetSensitivity(0.007);
	
				  Wait (1.0);
//	motor  [0]  ->SetVoltageRampRate(36);
//	motor  [1]  ->SetVoltageRampRate(36);
	motor[0]->ConfigNeutralMode(ThunderCANJaguar::kNeutralMode_Brake);
	motor[1]->ConfigNeutralMode(ThunderCANJaguar::kNeutralMode_Brake);
	encoder[0]  ->Start(   );
	encoder[1]  ->Start(   );
	thunder_gyro->Reset(   );
	speed_timer ->Reset(   );
	speed_timer ->Start(   );
	
}// \ThunderDrive::ThunderDrive()

ThunderDrive::~ThunderDrive()
{
	delete thunder_gyro;
	delete tracker     ;
	delete speed_timer       ;
	delete encoder[1]  ;
	delete encoder[0]  ;
	delete motor  [3]  ;
	delete motor  [2]  ;
	delete motor  [1]  ;
	delete motor  [0]  ;
}

float ThunderDrive::Inches_To_Revolutions(float inches)
{
	return (inches/WHEEL_CIRCUMFERENCE);
}

double ThunderDrive::Ticks_To_Inches(int ticks)
{
	//return (float)WHEEL_CIRCUMFERENCE*(((float)ticks)/(float)ENCODER_TICKS_PER_REV);
	return INCHES_PER_TICK*(double)ticks;
}

// is this redundant? see Tracker.cpp
bool ThunderDrive::Sensor_Active(int sensor)
{
	// sensor : 0 = left, 1 = center, 2 = right
	switch(sensor) {
	case 0: return tracker->GetLeft();
	case 1: return tracker->GetCenter();
	case 2: return tracker->GetRight();
	default: return false;
	}
}

// is this redundant? see Tracker.cpp
bool ThunderDrive::Light_Sensors_Match(bool * sensor)
{
	bool any = true;
	for (int a = 0; a < number_of_light_sensors; a++)
	{
		if (Sensor_Active(a) != sensor[a])
		{
			any = false;
			break;
		}
	}
	return any;
}

int ThunderDrive::Get_Encoder_Ticks(bool leftTrightF)
{
	return encoder[(int)(leftTrightF)]->Get();
}

float ThunderDrive::Get_Gyro_Angle()
{
	return thunder_gyro->GetAngle();
}

void ThunderDrive::Drive_Clear_Gyro()
{
	thunder_gyro->Reset();
}

// is this function way too long? I'm sorry, I lacked the time
// necessary to make it short
void ThunderDrive::Stay_On_Line(float& leftSpeed, float& rightSpeed)
{
	// ALL HAIL THE GLORIOUS AUTONOMOUS SOVEREIGN STATE OF ANTARCTICA!!!
	// (And most especially its wonderful inhabitants....)
	static int checkStop = 0;
	int work1, work2, work3, i, j;
	float speed;
	Tracker::TrackState state;
	leftSpeed = rightSpeed = 0; // by default, stop motors
	
	if(fromScratch == false) onLine = true; // no need to look for line
	
	/** NOTE -- make sure if we've aborted, we never get back here!! */
	// initially, we need to find the line; this loop takes rather
	// longer to complete as we attempt START_TRIES times to detect
	// the line (WILL THIS BREAK THINGS?); if we don't find it, onLine
	// will be false and drive_using_line will abort; otherwise
	// onLine will be true and we will enter standard line-tracking
	if(onLine == false) {
		for (i = 0; i < START_TRIES; i++) {
			tracker->PrintState(tracker->GetState());
			if(tracker->GetState()==Tracker::ON_LINE) { 
				onLine = true;
				break;
			}
		}
	}
	
	// prepare motor speeds to set depending on tracker state
	// uses first-generation speed ramping (based on distance)
	speed = Get_Max_Line_Speed();
	//printf("Speed: %f\n", speed);
	
	state = tracker->GetState();
//	tracker->PrintState(state); // debug
	switch(state) {
	case Tracker::ON_LINE:
		// on the line, we can continue to move
		leftSpeed = speed;
		rightSpeed = speed;
		break;
	case Tracker::TOO_LEFT:
		// too far left: lower right speed to turn right
		leftSpeed = speed;
		rightSpeed = speed*TURN_FACTOR;
		break;
	case Tracker::SEMI_LEFT:
		// a bit too left: turn right slightly
		leftSpeed = speed;
		rightSpeed = speed*TURN_FACTOR_SLIGHT;
		break;
	case Tracker::TOO_RIGHT:
		// too far right: lower left speed to turn left
		leftSpeed = speed*TURN_FACTOR;
		rightSpeed = speed;
		break;
	case Tracker::SEMI_RIGHT:
		// a bit too right: turn left slightly
		leftSpeed = speed*TURN_FACTOR_SLIGHT;
		rightSpeed = speed;
		break;
	case Tracker::WAY_OFF:
		//consult history and turn that way...
		// if we're consistently on line in the history (work2>=10)
		// we've probably gotten back in the history to our last
		// on-target state; work1 should now contain our last
		// off-target state
		// if instead we're off the line more than deemed safe
		// (work3>=MISS_TRIES) abort immediately!
		i = histpoint;
		j = work1 = work2 = work3 = 0;
		while(work2<10 and work3<MISS_TRIES) {					
			if(history[i]== Tracker::TOO_LEFT) work1--;
			if(history[i]== Tracker::SEMI_LEFT) work1--;
			if(history[i]== Tracker::TOO_RIGHT) work1++;
			if(history[i]== Tracker::SEMI_RIGHT) work1--;
			if(history[i]== Tracker::ON_LINE) work2++;
			if(history[i]== Tracker::WAY_OFF) work3++;
			i--;
			if(i<0) i = 1023;
			j++; // make sure we don't infinite loop
			if(j>1023) break;
		}
		if(work3 >= MISS_TRIES) { 
			// taking too long to correct, abort 
			printf("Way off too long, aborting\n");
			onLine = false;
			leftSpeed = 0;
			rightSpeed = 0;
			break;
		}
		if(work1 == 0) { // try to break tie with closest prev. data
			while(history[i]==Tracker::STOP or history[i]==Tracker::WYE or
					history[i]==Tracker::WAY_OFF or history[i]==Tracker::ON_LINE) {
				i--;
				if(i<0) i = 1023;
				j++; // prevent infinite loop
				if(j>1023) break;
			}
			if (history[i]==Tracker::TOO_LEFT or history[i]==Tracker::SEMI_LEFT)
				work1 = -1;
			else if(history[i]==Tracker::TOO_RIGHT or history[i]==Tracker::SEMI_RIGHT)
				work1 = 1;
		}
		if(work1 < 0) { // more to left, so turn back right
			printf("Way off and turning right\n");
			leftSpeed = speed;
			rightSpeed = speed*TURN_FACTOR;					
		} else if(work1 > 0) { // opposite
			printf("Way off and turning left\n");
			leftSpeed = speed*TURN_FACTOR;
			rightSpeed = speed;
		} else {
			printf("Way off, no way to tell how, aborting\n");
			onLine = false;
			// speeds will be zero
		}
		break;// \case Tracker::WAY_OFF
	case Tracker::WYE:
	case Tracker::STOP:
		// are we certain enough yet? if not, keep going
		checkStop++;
		if (checkStop < END_TRIES) break;
		// at the wye: stop first, then turn left (in Turn_Until_Line)
		// at stop bar: stop completely and finally
		// we've made it to our destination!
		endOfLine = true;
		// speeds will be zero
		if(state == Tracker::STOP) {
			printf("Stopping! CS:%d \n", checkStop);
			break;
		}
		printf("Stopping at wye!\n");
		break;
	} // \switch(state)

	// if we don't get a stop code, reset counter
	if(state!=Tracker::STOP and state!=Tracker::WYE)
		checkStop = 0;
	
	// finally, add to history
	history[histpoint] = state; 
	histpoint = ++histpoint % HIST_SIZE;
	
	// leftSpeed and rightSpeed will be returned by reference
}// \ThunderDrive::Stay_On_Line()

float ThunderDrive::Get_Max_Line_Speed() {
	float x_minus_h;
	// if path is too short to use method below, use a
	// plain quadratic path, with vertex halfway between start and
	// final constant MIN_SPEED segment (h=[autogoal-END_DISTANCE]/2);
	// speed MIN_SPEED at start and end; y = -F(x-h)^2 + k
	if(auto_goal < (2*ACCEL_DISTANCE + END_DISTANCE)) {
		if(auto_progress >= auto_goal-END_DISTANCE)
			return MIN_SPEED;
		x_minus_h = auto_progress - (auto_goal-END_DISTANCE)/2;
		return (speed_curve_k - ramp_factor*(x_minus_h*x_minus_h));
	}
	// but if we do have enough space...	
	// speed path starts at (0,MIN_SPEED) [(auto_progress,speed)]; then
	// rises quadratically to (ACCEL_DISTANCE, max_auto_speed) as 
	// a vertex; stays at max_auto_speed until auto_goal-ACCEL_DISTANCE-END_DISTANCE;
	// then declines quadratically to (auto_goal-END_DISTANCE, MIN_SPEED)
	// with a vertex of (auto_goal-ACCEL_DISTANCE-END_DISTANCE, max_auto_speed);
	// then stays at a constant MIN_SPEED until auto_goal is reached
	// both quadratic segments are y = -F(x-h)^2 + k,  where h,k is vertex
	// and F is the factor to slow the acceleration
	if(auto_progress < ACCEL_DISTANCE) // speeding up
		x_minus_h = auto_progress-ACCEL_DISTANCE;
	else if(auto_progress < auto_goal-END_DISTANCE-ACCEL_DISTANCE)
		x_minus_h = 0; // stay at max speed
	else if(auto_progress < auto_goal-END_DISTANCE)// slowing down
		x_minus_h = auto_progress-(auto_goal-END_DISTANCE-ACCEL_DISTANCE);
	else // constant at MIN_SPEED
		return MIN_SPEED;
	
	// k = max_auto_speed; F = ramp_factor
	return (max_auto_speed - ramp_factor*(x_minus_h*x_minus_h));
	
}
 
//left side negative = forward
//right side positive = forward
void ThunderDrive::Drive_Left_Right_Motors(float left_speed, float right_speed)
{
	l_speed     =  left_speed        ;
	r_speed     = right_speed        ;
	if(DIRECTION_CORRECTION < 0)
		r_speed-=DIRECTION_CORRECTION;
	else if
	  (DIRECTION_CORRECTION < 0)
		l_speed-=DIRECTION_CORRECTION;
	motor[0]->Set(- left_speed)      ;
	motor[1]->Set( right_speed)      ;
#if NUM_CIMS > 2
	motor[2]->Set(- left_speed)      ;
	motor[3]->Set( right_speed)      ;
#endif
}

void ThunderDrive::Drive_Set_Max_Auto_Speed(float max_speed)
{
	max_auto_speed = max_speed;
}

float ThunderDrive::Get_Last_Left_Speed()
{
	return l_speed;
}

float ThunderDrive::Get_Last_Right_Speed()
{
	return r_speed;
}

float ThunderDrive::Get_Current_Robot_Speed()
{
	return current_speed;
}

ThunderDrive::auto_task ThunderDrive::Get_Current_Auto_Task()
{
	return current_auto_task;
}

float ThunderDrive::Get_Auto_Progress()
{
	return auto_progress;
}

float ThunderDrive::Get_Auto_Start()
{
	return auto_start;
}

float ThunderDrive::Get_Auto_Goal()
{
	return auto_goal;
}

void ThunderDrive::Drive_Forward_Distance(float inches)
{
	auto_goal          = inches                 ;
	current_auto_task  = drive_dead_reckonning  ;
	current_auto_speed = 0                      ;
	auto_start         = 0                      ;
	auto_progress      = 0                      ;
	ramp_state         = 0                      ;
	                     encoder[0]  ->Reset   ();
	                     encoder[1]  ->Reset   ();
	auto_start_angle   = thunder_gyro->GetAngle();
}

void ThunderDrive::Drive_Turn_Angle(float angle)
{
	current_auto_task = turn_dead_reckonning          ;
	auto_start        = auto_progress 
					  = thunder_gyro->GetAngle()      ;
	auto_goal         = thunder_gyro->GetAngle()+angle;
}

void ThunderDrive::Drive_Turn_Until_Line(float degrees)
{
	//set current task to turn until line
	current_auto_task = turn_using_line;
	//set start to 0, and reset gyro...
	auto_start = 0;
	Drive_Clear_Gyro();
	//set progress to 0
	auto_progress = 0;
	//set goal to angle
	auto_goal = degrees;
	// haven't yet found line
	onLine = false;
	// compensate for overshoot by stopping "too early"
	// turning clockwise (right) => approaching line from left
	if(degrees > 0)
		turn_goal = Tracker::TOO_LEFT;
	else // and vice versa
		turn_goal = Tracker::TOO_RIGHT;
}

void ThunderDrive::Drive_Keep_On_Line(float inches, bool init)
{
	/**** NOTE: robot will overshoot target slightly, unfortunately... ****/
	/**** ~0.5-1 in.... ****/
	if(inches >= 9000.01);
	// set whether or not we're checking for line at start
	fromScratch = init;
	current_auto_task = drive_using_line; 
	auto_start = 0;
	//set progress to 0
	auto_progress = 0;
	//set goal to number of inches desired
	auto_goal = inches;
	// limit speed
	Drive_Set_Max_Auto_Speed(0.5);
	// ramp factor for Get_Max_Line_Speed; would give you
	// max_auto_speed at vertex of curve (ACCEL_DISTANCE):
	// y = -F(x-h)^2 + k, solving for F; (0,MIN_SPEED) is on curve...
	// MIN_SPEED = -F(0-h)^2 + k; F(h^2) = k - MIN_SPEED;
	// F = (k-MIN_SPEED)/(h^2)
	// [k = max_auto_speed; h = ACCEL_DISTANCE]
	ramp_factor = (max_auto_speed-MIN_SPEED)/(ACCEL_DISTANCE*ACCEL_DISTANCE);
	// value of k if we don't have enough distance to work with
	if(auto_goal < 2*ACCEL_DISTANCE+END_DISTANCE) {
		// y = -F(x-h)^2 + k, solving for k; (0,MIN_SPEED) is on curve...
		// MIN_SPEED = -F(0-h)^2 +k => k = F(h^2) + MIN_SPEED
		// [h = (autogoal-END_DISTANCE)/2]
		speed_curve_k = auto_goal-END_DISTANCE; // temp
		speed_curve_k = (speed_curve_k*speed_curve_k) / 4; // temp again
 		speed_curve_k = ramp_factor*speed_curve_k + MIN_SPEED ; // final 
	}
	// initialize history index
	histpoint = 0;
	// assume we're not yet on the line (to be safe)
	onLine = false;
	// and we're not at the end yet either
	endOfLine = false;
	// fill history with a nonsensical value to avoid garbage values
	// breaking things, while not confusing the algorithm
	for(int i = 0; i<1024; i++)
		history[i] = 10;
	encoder[0]->Reset();
	encoder[1]->Reset();
}// \ThunderDrive::Drive_Keep_On_Line()


void ThunderDrive::Apply_Ramping(float * current_speed, float progress, float goal, float up_rate, float down_rate, float max, float stop_up, float start_down, float * L, float * R)
{
	switch(ramp_state)
	{
	case 0:
		*L = *R = *current_speed+up_rate*GET_SIGN(goal);
		if((stop_up > fabs(goal) && progress >= fabs(goal)/8) || progress >= fabs(goal)/2) 
					ramp_state = 2;
		else if(fabs(*current_speed) >= max)
			ramp_state = 1;
		break;
	case 1:
		*L = *R = max*GET_SIGN(goal);
		if(progress >= fabs(goal)-start_down)
			ramp_state = 2;
		break;
	case 2:
		*L = *R = *current_speed-down_rate*GET_SIGN(goal);
		break;
	}
	*current_speed = *L;
}


void ThunderDrive::Apply_Gyro_Correction(float target, float current, float factor, float * L, float * R)
{
	*R += (current-target)/factor;
	*L -= (current-target)/factor;
}

void ThunderDrive::Dead_Reckonning_Debug()
{
	if(!DEAD_RECKONNING_DEBUG) return;
	if(speed_timer->Get() > 0.75 || Drive_Current_Auto_Task_Done())
	{
		speed_timer->Reset()         ;
					
		float this_distance = Ticks_To_Inches(((encoder[0]->Get()+encoder[1]->Get())/2))-last_distance;
		last_distance       = Ticks_To_Inches(((encoder[0]->Get()+encoder[1]->Get())/2))              ;
		current_speed       = this_distance/speed_timer->Get();  
		printf(
				"\nMotor:   |L:%f|R:%f\nEncoder: |L:%i|R:%i|\nDistance:|L:%f|R:%f|\nAngle:   |Actual:%f|Start:%f|Diff:%f|\nState:   |goal:%f|progress:%f|ramp:%i\n", 
				(double)l_speed, (double)r_speed,
				encoder[0]->Get(), encoder[1]->Get(),
				(double)Ticks_To_Inches(encoder[0]->Get()), (double)Ticks_To_Inches(encoder[1]->Get()),
				(double)thunder_gyro->GetAngle(), (double)auto_start_angle, (double)(thunder_gyro->GetAngle()-auto_start_angle),
				(double)auto_goal, (double)auto_progress, ramp_state  
		);
	}
}

void ThunderDrive::Drive_Process_Auto()
{
	float L,R,angle_diff                                                  ;
	switch (current_auto_task)
	{
	case drive_dead_reckonning:
		
		auto_progress = Ticks_To_Inches(
			//GET_LESSER
				(encoder[0]->Get()+encoder[1]->Get())/2)                     ;
		//positive diff: need to turn CCW (-,+), negative diff: need to turn CW (+,-)
		angle_diff    = thunder_gyro->GetAngle()-auto_start_angle         ;
		/*	switch(ramp_state)
				{
				case 0:
					L = R = current_auto_speed+ramp_rate*GET_SIGN(auto_goal);
					if((Ticks_To_Inches((int)(max_auto_speed/ramp_rate)) > fabs(auto_goal) && auto_progress >= fabs(auto_goal)/4) || auto_progress >= fabs(auto_goal)/2) ramp_state = 2;
					else if(fabs(current_auto_speed) >= max_auto_speed) ramp_state = 1;
					break;
				case 1:
					L = R = max_auto_speed*GET_SIGN(auto_goal);
					//slope = rise/run, run = slope/rise, ramp_distance = max_auto_speed/ramp_rate
					if(auto_progress >= fabs(auto_goal-Ticks_To_Inches((int)(max_auto_speed/(ramp_rate)))))
						ramp_state = 2;
					break;
				case 2:
					L = R  = current_auto_speed-5*ramp_rate*GET_SIGN(auto_goal);                         ;
					break;
				}
		current_auto_speed = L;*/
		//R += angle_diff/45;
		//L -= angle_diff/45;
		Apply_Ramping(&current_auto_speed, auto_progress, auto_goal, ramp_rate, 5*ramp_rate, max_auto_speed,
				Ticks_To_Inches((int)(max_auto_speed/ramp_rate)), 
				(max_auto_speed*9)*Ticks_To_Inches((int)(max_auto_speed/(ramp_rate))), &L, &R);
//		CLAbhMP_VALUE            (L, GET_SIGN(auto_goal)*0.19, (auto_goal<0));
//		CLAMP_VALUE            (R, GET_SIGN(auto_goal)*0.19, (auto_goal<0));
		Apply_Gyro_Correction(auto_start_angle, thunder_gyro->GetAngle(), 10, &L, &R);
		CLAMP_VALUE(L, GET_SIGN(auto_goal)*0.175, (auto_goal<0));
		CLAMP_VALUE(R, GET_SIGN(auto_goal)*0.175, (auto_goal<0));
		if(Drive_Current_Auto_Task_Done())
		{
			L = R = 0;
		}
		//CLAMP_VALUE            (L, GET_SIGN(auto_goal)*0.1, (auto_goal<0));
		//CLAMP_VALUE            (R, GET_SIGN(auto_goal)*0.1, (auto_goal<0));
		
		if(Ticks_To_Inches(encoder[0]->Get()) > fabs(auto_goal)) L = 0;
		if(Ticks_To_Inches(encoder[1]->Get()) > fabs(auto_goal)) R = 0;
		Drive_Left_Right_Motors(L, R);
		Dead_Reckonning_Debug();
		/*if(Drive_Current_Auto_Task_Done())
		{
				L = R = 0;
				printf("\nFinal State:")
		}
		if(speed_timer->Get() > 0.75 || Drive_Current_Auto_Task_Done())
		{
			speed_timer->Reset()         ;
						
			float this_distance = Ticks_To_Inches(((encoder[0]->Get()+encoder[1]->Get())/2))-last_distance;
			last_distance       = Ticks_To_Inches(((encoder[0]->Get()+encoder[1]->Get())/2))              ;
			current_speed       = this_distance/speed_timer->Get();  
			printf(
					"\nMotor:   |L:%f|R:%f\nEncoder: |L:%i|R:%i|\nDistance:|L:%f|R:%f|\nAngle:   |Actual:%f|Start:%f|Diff:%f|\nState:   |goal:%f|ramp:%i\n", 
					(double)L, (double)R,
					encoder[0]->Get(), encoder[1]->Get(),
					(double)Ticks_To_Inches(encoder[0]->Get()), (double)Ticks_To_Inches(encoder[1]->Get()),
					(double)thunder_gyro->GetAngle(), (double)auto_start_angle, (double)angle_diff,
					(double)auto_goal, ramp_state  
			);
		}*/

		break; // \case drive_dead_reckonning
		
	case drive_using_line:
		//set progress to 1 if sensors are reading the end of the line
		auto_progress = Ticks_To_Inches((encoder[0]->Get()+encoder[1]->Get())/2);
		//run staying on line algorithm
		Stay_On_Line(L,R);
		//printf("Progress: %f, Goal: %f\n", auto_progress, auto_goal);
		// failsafe to make sure we stop immediately if we've gone too far
		if(auto_progress >= (auto_goal - 5)) {
			printf("Drive Reached goal, stopping motors\n");
			motor[0]->Set(0);
			motor[1]->Set(0);			
#if NUM_CIMS > 2
			motor[2]->Set(0);
			motor[3]->Set(0);
#endif
			break; // with motor speeds zero, so we don't move even further
		}
		//run motors with calculated values
		Drive_Left_Right_Motors(L, R);
		break;// \case drive_using_line
	case turn_dead_reckonning:
		
		auto_progress = thunder_gyro->GetAngle()                          ;
		switch(ramp_state)
		{
		case 0:
			L = current_auto_speed+ramp_rate*GET_SIGN(auto_goal);
			R = -current_auto_speed-ramp_rate*GET_SIGN(auto_goal);
			if(fabs(current_auto_speed) >= max_auto_speed/2.5) ramp_state = 1;
			break;
		case 1:
			L =  max_auto_speed/2.5*GET_SIGN(auto_goal);
			R = -max_auto_speed/2.5*GET_SIGN(auto_goal);
			if(auto_progress >= fabs(auto_goal)-25) ramp_state = 2;
			break;
		case 2:
			L = current_auto_speed-ramp_rate*1.5*GET_SIGN(auto_goal);
			R = -current_auto_speed+ramp_rate*1.5*GET_SIGN(auto_goal);
			break;
		}		
		current_auto_speed = L;
		CLAMP_VALUE            (L,
				 GET_SIGN(auto_goal)*0.25, (auto_goal<0))                  ;
		CLAMP_VALUE            (R,
				-GET_SIGN(auto_goal)*0.25, (auto_goal>0))                  ;
		//printf("L:%f|R:%f|A:%f|G:%f|S:%i\n", (double)L, (double)R, (double)thunder_gyro->GetAngle(), (double)auto_goal, ramp_state);
		Drive_Left_Right_Motors(L, R)                                     ;
		
		
		break; // \case turn_dead_reckonning
		
	case turn_using_line:
		//set onLine to true and stop motors if we're on line
		if(tracker->GetState()==turn_goal) {
			onLine = true;
			motor[0]->Set(0);
			motor[1]->Set(0);
#if NUM_CIMS > 2
			motor[2]->Set(0);
			motor[3]->Set(0);			
#endif
			break;
		}
		// update progress
		auto_progress = thunder_gyro->GetAngle();
		// if we've turned max angle, stop immediately
		if(fabs(auto_progress) > fabs(auto_goal)) {
			motor[0]->Set(0);
			motor[1]->Set(0);
#if NUM_CIMS > 2
			motor[2]->Set(0);
			motor[3]->Set(0);
#endif
			break;
		}
		//turn motors on in opposite directions depending upon direction you're turning
		L = R = TURN_SPEED;
		if(auto_goal > 0) // turning clockwise (right)
			R = -R;
		else // turning counterclockwise (left)
			L = -L;
		Drive_Left_Right_Motors(L, R);
		// debug...
		//printf("Gyro:%f -- ",auto_progress);
		//tracker->PrintState(tracker->GetState());
		break;
	case none:
		break;
	default:
		throw "bad task in ThunderDrive::Process_Auto"                    ;
		break;
	}// \switch(current_auto_task)
}// \ThunderDrive::Drive_Process_Auto()

bool ThunderDrive::Drive_Current_Auto_Task_Done()
{
// note these conditions are not nearly correct for drive_using_line
// or for turn_using_line
	switch (current_auto_task)
	{
	case drive_dead_reckonning:
		//tollerance of 1.5 inch.
		return (auto_progress                 >= fabs(auto_goal)-1.5);
		break;
	case turn_dead_reckonning:
		return (fabs((auto_goal-auto_progress)) <  5        );
		break;
	case drive_using_line:
		// if we've lost the line, DON'T give up control
		// hold on to it greedily lest other subtasks destroy the robot!!!!!
		if(!onLine) {
			printf("Off line, aborting...\n");
			return false;
		}
//		sensors = new bool[number_of_light_sensors];
		// otherwise, finish either if we've driven far enough
		// or we've reached the end of the line, or the wye
		if(endOfLine) printf("End of line, quitting...\n");
		if(endOfLine || fabs(auto_goal-auto_progress) <  5) {
			current_auto_task = none;
			return true;
		}
		return false;
		break;
	case turn_using_line:
//		sensors = new bool[number_of_light_sensors];
//		return Light_Sensors_Match(sensors);
		// found line?
		if(onLine == true) {
			printf("Found line; ready to continue!\n");
			return true;
		}
		if(fabs(auto_progress) > fabs(auto_goal))
			printf("Turned max angle, quitting...\n");
		return fabs(auto_progress) > fabs(auto_goal);
		break;
	case none:
		return true;
		break;
	default:
		throw "bad task in ThunderDrive::Current_Auto_Task";
		break;
	}// \swtich(current_auto_task)
}// \ThunderDrive::Drive_Current_Auto_Task_Done()


void ThunderDrive::Set_Coast(bool coast)
{
	if(coast)
	{
		motor[0]->ConfigNeutralMode(ThunderCANJaguar::kNeutralMode_Coast);
		motor[1]->ConfigNeutralMode(ThunderCANJaguar::kNeutralMode_Coast);
#if NUM_CIMS > 2
		motor[2]->ConfigNeutralMode(ThunderCANJaguar::kNeutralMode_Coast);
		motor[3]->ConfigNeutralMode(ThunderCANJaguar::kNeutralMode_Coast);
#endif
	}
	if(!coast)
	{
		motor[0]->ConfigNeutralMode(ThunderCANJaguar::kNeutralMode_Brake);
		motor[1]->ConfigNeutralMode(ThunderCANJaguar::kNeutralMode_Brake);
#if NUM_CIMS > 2
		motor[2]->ConfigNeutralMode(ThunderCANJaguar::kNeutralMode_Brake);
		motor[3]->ConfigNeutralMode(ThunderCANJaguar::kNeutralMode_Brake);
#endif
	}
}

void ThunderDrive::Drive_Stop_Auto_Task()
{
	current_auto_task = none;
}
