#include "Resources.h"
#include "ThunderLight.h"

ThunderLight::ThunderLight():
	RedLight(DIG_LIGHT_RED),
	WhiteLight(DIG_LIGHT_WHITE),
	BlueLight(DIG_LIGHT_BLUE),
	GreenLight(DIG_LIGHT_GREEN)
{
	RedLight.SetPWMRate(120.0f);
	RedLight.EnablePWM(0);
	WhiteLight.EnablePWM(0);
	BlueLight.EnablePWM(0);
	GreenLight.EnablePWM(0);
	DigitalOutputs[0] = &GreenLight;
	DigitalOutputs[1] = &RedLight;
	DigitalOutputs[2] = &WhiteLight;
	DigitalOutputs[3] = &BlueLight;
	srandom((unsigned long)GetTime());
	startTime = GetTime();
}

ThunderLight::~ThunderLight()
{
}

void ThunderLight::DoPrematchSignaling(PrematchState state) {
	switch (state) {
	case PREMATCH_GYROCALIB:
		RedLight.SetPWMRate(4.0f);
		RedLight.UpdateDutyCycle(0.5f);
		break;
	case PREMATCH_GYRODONE:
		WhiteLight.UpdateDutyCycle(0.5f);
		BlueLight.UpdateDutyCycle(0.5f);
		GreenLight.UpdateDutyCycle(0.5f);
		RedLight.UpdateDutyCycle(0.5f);
		break;
	case PREMATCH_COMPLETE:
		RedLight.SetPWMRate(120.0f);
		WhiteLight.UpdateDutyCycle(0);
		BlueLight.UpdateDutyCycle(0);
		GreenLight.UpdateDutyCycle(0);
		RedLight.UpdateDutyCycle(0);
		break;
	}
}

void ThunderLight::SetColors(bool red, bool white, bool blue, bool green)
{
	RedLight.UpdateDutyCycle((float)red);
	WhiteLight.UpdateDutyCycle((float)white);
	BlueLight.UpdateDutyCycle((float)blue);
	GreenLight.UpdateDutyCycle((float)green);
}

void ThunderLight::LightChaseUp()
{
	//offset the time to get nice, small numbers
	double curTime = GetTime() - startTime;
	
	if(curTime >= 2)  //reset time if over 2 seconds
	{
		startTime = GetTime();
		curTime = GetTime() - startTime; //recalculate curTime
	}
	
	
	//sets the digital output in the array high 
	//if it is equal to the seconds
	//All others are low
	for(int i = 0; i <= 3; i++)
	{
		if(i == floor(curTime * 2))
			DigitalOutputs[i]->UpdateDutyCycle(1);
		else
			DigitalOutputs[i]->UpdateDutyCycle(0);
	}
}

void ThunderLight::LightChaseDown()
{
	//offset the time to get nice, small numbers
	double curTime = GetTime() - startTime;
	
	if(curTime >= 2)  //reset time if over 2 seconds
	{
		startTime = GetTime();
		curTime = GetTime() - startTime; //recalculate curtime
	}
		
	
	//sets the digital output in the array high 
	//if it is equal to the seconds
	//All others are low
	//Does this in the reverse pattern as the above function
	for(int i = 0; i <= 3; i++)
	{
		if(i == floor(curTime * 2))
			DigitalOutputs[3 - i]->UpdateDutyCycle(1);
		else
			DigitalOutputs[3 - i]->UpdateDutyCycle(0);
	}
}


void ThunderLight::LightChaseUpDown()
{
	//offset the time to get nice, small numbers
	double curTime = GetTime() - startTime;
	
	if(curTime >= 6)  //reset time if over 6 seconds
	{
		startTime = GetTime();
		curTime = GetTime() - startTime; //recalculate curtime
	}
	
	//sets the digital output in the array high 
	//if it is equal to the seconds
	//All others are low
	switch ((int)curTime){
	case 0:
		RedLight.UpdateDutyCycle(0);
		GreenLight.UpdateDutyCycle(1);
		break;
	case 1:
		GreenLight.UpdateDutyCycle(0);
		RedLight.UpdateDutyCycle(1);
		break;
	case 2:
		RedLight.UpdateDutyCycle(0);
		WhiteLight.UpdateDutyCycle(1);
		break;
	case 3:
		WhiteLight.UpdateDutyCycle(0);
		BlueLight.UpdateDutyCycle(1);
		break;
	case 4:
		BlueLight.UpdateDutyCycle(0);
		WhiteLight.UpdateDutyCycle(1);
		break;
	case 5:
		WhiteLight.UpdateDutyCycle(0);
		RedLight.UpdateDutyCycle(1);
		break;
	default:
		break;
	}
}
	


/*
Hello, ladies, look at your man, now back to me, 
now back at your man, now back to me. 
Sadly, he isnt me, but if he stopped using 
ladies scented body wash and switched to Old Spice, 
he could smell like hes me. Look down, back up, where are you? 
Youre on a boat with the man your man could smell like. 
Whats in your hand? Back at me. 
I have it. Its an oyster with two tickets to that thing you love.
Look again, the tickets are now diamonds. 
Anything is possible when your man smells like Old Spice 
and not a lady.

Im on a horse.
*/

void ThunderLight::LightRandom()
{
	//offset the time to get nice, small numbers
	double curTime = GetTime() - startTime;
	
	if(curTime >= .5)  //reset time if over .5 seconds
	{
		startTime = GetTime();
		curTime = GetTime() - startTime; //recalculate curTime
		//Pick a random light from 0 - 3
		currentLight = (int)floor(((float)random() / ULONG_MAX) * 4); 
	}
	
	for(int i = 0; i <= 3; i++)
	{
		if(i == currentLight)
			DigitalOutputs[i]->UpdateDutyCycle(1);
		else
			DigitalOutputs[i]->UpdateDutyCycle(0);
	}
		
}

void ThunderLight::FadeInOut()
{
	//offset the time to get nice, small numbers
	double curTime = GetTime() - startTime;
	
	if(curTime >= 5)  //reset time if over 2 seconds
	{
		startTime = GetTime();
		curTime = GetTime() - startTime; //recalculate curTime
	}
	
	for(int i = 0; i <= 3; i++)
	{
		if(curTime < 1.9)
			DigitalOutputs[i]->UpdateDutyCycle(curTime / 2);
		else if(curTime < 3.1)
			DigitalOutputs[i]->UpdateDutyCycle(.95);
		else
			DigitalOutputs[i]->UpdateDutyCycle((5 - curTime)/2);
	}
}

//kjfdsiuvnfnnv;rew.bfdsvdnvijv;oiyvdskn.knb;ljvcoiu


void ThunderLight::FadeUpDown()
{
	//offset the time to get nice, small numbers
	double curTime = GetTime() - startTime;
	
	if(curTime >= 6)  //reset time if over 6 seconds
	{
		startTime = GetTime();
		curTime = GetTime() - startTime; //recalculate curtime
	}
	
	//sets the digital output in the array high 
	//if it is equal to the seconds
	//All others are low
	switch ((int)curTime){
	case 0:
		RedLight.UpdateDutyCycle(1 - curTime);
		GreenLight.UpdateDutyCycle(curTime);
		break;
	case 1:
		GreenLight.UpdateDutyCycle(2 - curTime);
		RedLight.UpdateDutyCycle(curTime - 1);
		break;
	case 2:
		RedLight.UpdateDutyCycle(3 - curTime);
		WhiteLight.UpdateDutyCycle(curTime - 2);
		break;
	case 3:
		WhiteLight.UpdateDutyCycle(4 - curTime);
		BlueLight.UpdateDutyCycle(curTime -  3);
		break;
	case 4:
		BlueLight.UpdateDutyCycle(5 - curTime);
		WhiteLight.UpdateDutyCycle(curTime - 4);
		break;
	case 5:
		WhiteLight.UpdateDutyCycle(6 - curTime);
		RedLight.UpdateDutyCycle(curTime - 5);
		break;
	default:
		break;
	}
}

void ThunderLight::FireLaser()
{
	double curTime = GetTime() - startTime; //The time since the pattern has started

	if(curTime < 5)
	{
		GreenLight.UpdateDutyCycle((curTime/5) * ( 1 + ((float)random() / ULONG_MAX)/7));
	}
	else if(curTime < 5.3)
	{
		if(curTime < 5.1)
		{
			GreenLight.UpdateDutyCycle(.9);
			RedLight.UpdateDutyCycle(.9);
		}
		else if(curTime < 5.2)
		{
			GreenLight.UpdateDutyCycle(.8);
			RedLight.UpdateDutyCycle(.8);
			WhiteLight.UpdateDutyCycle(.8);
		}
		else
		{
			GreenLight.UpdateDutyCycle(.7);
			RedLight.UpdateDutyCycle(.7);
			WhiteLight.UpdateDutyCycle(.7);
			BlueLight.UpdateDutyCycle(.7);
		}
	}
	else if(curTime < 6.3)
	{
		GreenLight.UpdateDutyCycle(.7 - (.7 * (curTime - 5.3)));
		RedLight.UpdateDutyCycle(.7 - (.7 * (curTime - 5.3)));
		WhiteLight.UpdateDutyCycle(.7 - (.7 * (curTime - 5.3)));
		BlueLight.UpdateDutyCycle(.7 - (.7 * (curTime - 5.3)));
	}
	else if(curTime < 7)
	{
		//Do nothing until loop, keep all lights off
	}
	else
	{
		//Reset time
		startTime = GetTime();
	}
}

void ThunderLight::FireLaser2()
{
	double curTime = GetTime() - startTime; //The time since the pattern has started

	if (curTime <3)
	{
		GreenLight.UpdateDutyCycle((curTime/3) * ( 1 + ((float)random() / ULONG_MAX)/7));
		RedLight.UpdateDutyCycle((curTime/9) * ( 1 + ((float)random() / ULONG_MAX)/7));
	}
	else if(curTime < 3.33)
	{
		if(curTime < 3.2)
		{
			RedLight.UpdateDutyCycle(.33 + ((curTime - 3) / .2) * .67);
			WhiteLight.UpdateDutyCycle(((curTime - 3) / .2) * .75);
			BlueLight.UpdateDutyCycle(((curTime - 3) / .2) * .60);
		}
		else if(curTime < 3.3)
		{
			WhiteLight.UpdateDutyCycle((((curTime - 3.2) / .1) * 25) + .75);
			BlueLight.UpdateDutyCycle((((curTime - 3.2) / .1) * .3) + .6);
		}
		else if(curTime < 3.33)
		{
			BlueLight.UpdateDutyCycle((((curTime - 3.3) / .03) * .1) + .9);
		}
	}
	else if(curTime < 4.68)
	{
		if(curTime < 3.83)
		{
			RedLight.UpdateDutyCycle((((3.83 - curTime) / .5) * .5) + .5);
			WhiteLight.UpdateDutyCycle((((3.83 - curTime) / .5) * .8) + .2);
			BlueLight.UpdateDutyCycle((3.83 - curTime) / .5);
		}
		else if(curTime < 4.03)
		{
			WhiteLight.UpdateDutyCycle(4.03 - curTime);
			RedLight.UpdateDutyCycle((((4.03 - curTime) / .2) * .3) + .2);
			GreenLight.UpdateDutyCycle((((4.03 - curTime) / .2) * .5) + .5);
		}
		else if(curTime < 4.13)
		{
			RedLight.UpdateDutyCycle(((4.13 - curTime) / .1) * .2);
			GreenLight.UpdateDutyCycle((((4.13 - curTime) / .1) * .3) + .2);
		}
		else if(curTime < 4.18)
		{
			GreenLight.UpdateDutyCycle(((4.18 - curTime) / .05) * .2);
		}
	}
	else if(curTime < 4.68)
	{
		//Do nothing until loop, keep all lights off
	}
	else
	{
		startTime = GetTime();
	}
}

void ThunderLight::WarningFlash()
{
	//offset the time to get nice, small numbers
	double curTime = GetTime() - startTime;
	
	if(curTime >= 1)  //reset time if over 1 second
	{
		startTime = GetTime();
		curTime = GetTime() - startTime; //recalculate curTime
	}
	
	
	//sets the digital output in the array high 
	//if it is equal to the seconds
	//All others are low
	if(curTime < .5)
	{
		BlueLight.UpdateDutyCycle(1);
		WhiteLight.UpdateDutyCycle(1);
		RedLight.UpdateDutyCycle(0);
		GreenLight.UpdateDutyCycle(0);	
	}
	else
	{
		BlueLight.UpdateDutyCycle(0);
		WhiteLight.UpdateDutyCycle(0);
		RedLight.UpdateDutyCycle(1);
		GreenLight.UpdateDutyCycle(1);	
	}
}

void ThunderLight::SinePulse()
{
    double curTime = GetTime() - startTime;
    double frac = curTime / 4.0;
    
    if (frac > 1.0) {
        frac = 0;
        startTime = GetTime();
    }
    
    double v = sin(frac * 3.14159);
    if (v < 0) v = 0;
    if (v >= 0.95)
        v = 0.95;
    for (int i = 0; i <= 3; i++)
        DigitalOutputs[i]->UpdateDutyCycle(v);
}

void ThunderLight::SineFlash()
{
    static int state = 0;
    
    double curTime = GetTime() - startTime;

    double frac = curTime / 0.5;

    switch (state) {
        case 0:
        {
            double last;
            for(int i = 0; i <= 3; i++)
            {
                //double offset = i*10 / (4.0 );
                double offset = i*0.3;
                double v;
                v = cos((frac - offset) * 3.14159 * 1);
                if (v<0)
                    v=0;
                if (v >= 0.95f)
                    v=0.95;
                DigitalOutputs[i]->UpdateDutyCycle(v);
                last = v;
            }
            if (last == 0.95) {
                state=2;
                startTime=GetTime();
            }
            break;
        }
        case 2:
        {
            double first;
            for(int i = 3; i >= 0; i--)
            {
                //double offset = (3-i)*10 / (4.0 );
                double offset = (3-i)*0.3;
                double v;
                v = cos((frac - offset) * 3.14159 * 1);
                if (v<0)
                    v=0;
                if (v >= 0.95f)
                    v=0.95;
                DigitalOutputs[i]->UpdateDutyCycle(v);
                first = v;
            }
            if (first == 0.95) {
                //state++;
                state=0;
                startTime = GetTime();
            }
            break;
        }
    }
    
}

#if 0
void ThunderLight::SineFlash()
{
    static int state = 0;
    
    double curTime = GetTime() - startTime;
    if (false && curTime >= 6) {
        startTime = GetTime();
        curTime = 0;
    }

    double frac = curTime / 5.0;

    switch (state) {
        case 0:
        {
            double last;
            for(int i = 0; i <= 3; i++)
            {
                double offset = i / (4.0 * 10);
                double v;
                v = sin((frac - offset) * 3.14159 * 1);
                if (v<0)
                    v=0;
                if (v >= 0.95f)
                    v=0.95;
                DigitalOutputs[i]->UpdateDutyCycle(v);
                last = v;
            }
            if (last == 0.95)
                state++;
            break;
        }
        case 1:
        {
            double last;
            for(int i = 0; i <= 2; i++)
            {
                double offset = i / (4.0 * 10);
                double v;
                v = sin((frac - offset) * 3.14159 * 1);
                if (v<0)
                    v=0;
                if (v >= 0.95f)
                    v=0.95;
                DigitalOutputs[i]->UpdateDutyCycle(v);
                last = v;
            }
            if (last < 0.01) {
                state++;
                startTime = GetTime()-0.25;
            }
            break;
        }
        case 2:
        {
            double first;
            for(int i = 3; i >= 0; i--)
            {
                double offset = (3-i) / (4.0 * 10);
                double v;
                v = sin((frac - offset) * 3.14159 * 1);
                if (v<0)
                    v=0;
                if (v >= 0.95f)
                    v=0.95;
                DigitalOutputs[i]->UpdateDutyCycle(v);
                first = v;
            }
            if (first == 0.95)
                state++;
            break;
        }
        case 3:
        {
            double first;
            for(int i = 3; i >= 1; i--)
            {
                double offset = (3-i) / (4.0 * 10);
                double v;
                v = sin((frac - offset) * 3.14159 * 1);
                if (v<0)
                    v=0;
                if (v >= 0.95f)
                    v=0.95;
                DigitalOutputs[i]->UpdateDutyCycle(v);
                first = v;
            }
            if (first < 0.01) {
                state = 0;
                startTime = GetTime()-0.25;
            }
            break;
        }
    }
    
}
#endif

void ThunderLight::Dim()
{
	BlueLight.UpdateDutyCycle(0.3);
	WhiteLight.UpdateDutyCycle(0.3);
	RedLight.UpdateDutyCycle(0.3);
	GreenLight.UpdateDutyCycle(0.3);		
}

void ThunderLight::Binary()
{
	//offset the time to get nice, small numbers
	double curTime = GetTime() - startTime;
	
	if(curTime >= 3)  //reset time if over 1 second
	{
		startTime = GetTime();
		curTime = GetTime() - startTime; //recalculate curTime
	}
	
	if(curTime < .5)
	{
		GreenLight.UpdateDutyCycle(0);
		RedLight.UpdateDutyCycle(0);
		WhiteLight.UpdateDutyCycle(0);
		BlueLight.UpdateDutyCycle(1);
	}
	else if (curTime < .6)
	{
		GreenLight.UpdateDutyCycle(0);
		RedLight.UpdateDutyCycle(0);
		WhiteLight.UpdateDutyCycle(0);
		BlueLight.UpdateDutyCycle(0);
	}
	else if (curTime < 1.1)
	{
		GreenLight.UpdateDutyCycle(0);
		RedLight.UpdateDutyCycle(1);
		WhiteLight.UpdateDutyCycle(0);
		BlueLight.UpdateDutyCycle(1);
	}
	else if (curTime < 1.2)
	{
		GreenLight.UpdateDutyCycle(0);
		RedLight.UpdateDutyCycle(0);
		WhiteLight.UpdateDutyCycle(0);
		BlueLight.UpdateDutyCycle(0);
	}
	else if (curTime < 1.7)
	{
		GreenLight.UpdateDutyCycle(0);
		RedLight.UpdateDutyCycle(0);
		WhiteLight.UpdateDutyCycle(0);
		BlueLight.UpdateDutyCycle(1);
	}
	else if (curTime < 1.8)
	{
		GreenLight.UpdateDutyCycle(0);
		RedLight.UpdateDutyCycle(0);
		WhiteLight.UpdateDutyCycle(0);
		BlueLight.UpdateDutyCycle(0);
	}
	else if (curTime < 2.3)
	{
		GreenLight.UpdateDutyCycle(0);
		RedLight.UpdateDutyCycle(0);
		WhiteLight.UpdateDutyCycle(0);
		BlueLight.UpdateDutyCycle(1);
	}
	else
	{
		GreenLight.UpdateDutyCycle(0);
		RedLight.UpdateDutyCycle(0);
		WhiteLight.UpdateDutyCycle(0);
		BlueLight.UpdateDutyCycle(0);
	}
}

void ThunderLight::ClearLights()
{
	RedLight.UpdateDutyCycle(0);
	WhiteLight.UpdateDutyCycle(0);
	BlueLight.UpdateDutyCycle(0);
	GreenLight.UpdateDutyCycle(0);	
	startTime = GetTime();
}

void ThunderLight::OnOffFlash()
{
	//offset the time to get nice, small numbers
	double curTime = GetTime() - startTime;
	
	if(curTime >= 0.25)
	{
		startTime = GetTime();
		curTime = 0;
	}
	
	if (curTime < 0.25/2) {
		// Turn on
		RedLight.UpdateDutyCycle(1);
		WhiteLight.UpdateDutyCycle(1);
		BlueLight.UpdateDutyCycle(1);
		GreenLight.UpdateDutyCycle(1);	
	} else {
		// Turn off
		RedLight.UpdateDutyCycle(0);
		WhiteLight.UpdateDutyCycle(0);
		BlueLight.UpdateDutyCycle(0);
		GreenLight.UpdateDutyCycle(0);	
	}
}
