#ifndef THUNDER_LIGHT_H
#define THUNDER_LIGHT_H

#include "Resources.h"
#include "Math.h"
#include "Random.h"

class ThunderLight
{
public:
	ThunderLight();
	~ThunderLight();
	//Put functions hither

	typedef enum {
		PREMATCH_GYROCALIB,		// Calibrating gyro
		PREMATCH_GYRODONE,		// Done calibrating gyro
		PREMATCH_COMPLETE		// Done with all prematch stuff - go to normal operation
	} PrematchState;
	// light sequences for pre-match states to show drivers what is going on
	//  while program is coming up
	// If you use this, make sure to call at least once with PREMATCH_COMPLETE
	//  before using any other functions in this class
	void DoPrematchSignaling(PrematchState state);

	//Allows you to control each light individually
	//True for on, false for off
	void SetColors(bool red, bool white, bool blue, bool green);
	

	
	//Makes the lights go in an upwards pattern
	//Call constantly when you want the effect
	void LightChaseUp();
	
	//Makes the lights move in a downwards pattern
	//Call constantly when you want the effect
	void LightChaseDown();
	
	//Makes the lights mive up then down
	//Call constantly when you want the effect
	void LightChaseUpDown();
	
	//Makes the lights flash in a random pattern
	//Call constantly when you want the effect
	void LightRandom();
	
	//Fades the lights in then out
	//Call constatnly when you want the effect
	void FadeInOut();
	
	//Up/down chase sequence with fading
	//Call constantly when you want the effect
	void FadeUpDown();
	
	//Plays the "firing laser" lightshow
	//Call constantly when you want the effect
	void FireLaser();
	
	//Called same as above
	//Only difference is different pattern
	void FireLaser2();
	
	//Flashes the top two lights, then the bottom two
	//call constantly when you want the effect
	void WarningFlash();
	
	void SinePulse();
	void SineFlash();
	void OnOffFlash();
	
	// Dim constantly
	void Dim();

	//Flashes 0001, 0101, 0001, 0001
	void Binary();
	
	//Clears the lights
	//call when switching patterns
	void ClearLights();
	
private:
	DigitalOutput RedLight;
	DigitalOutput WhiteLight;
	DigitalOutput BlueLight;
	DigitalOutput GreenLight;
	
	DigitalOutput* DigitalOutputs[4];
	
	double startTime;
	int currentLight;
	
};

#endif
