#include "Resources.h"
#include "ThunderManip.h"

ThunderManip::ThunderManip()
{
	upperMotor = new ThunderCANJaguar(CANID_MANIP_UPPER);
	lowerMotor = new ThunderCANJaguar(CANID_MANIP_LOWER);
	manipBannerSensor = new AnalogChannel(ANA_MANIP_GOT_IT);
	TwistSpeed = 0;
	InOutSpeed = 0;
	AutoGoalTime = 0;
	AutoOutTime = 0;
	BannerBroken = false;
}

ThunderManip::~ThunderManip()
{
	delete manipBannerSensor;
	delete lowerMotor;
	delete upperMotor;
}

void ThunderManip::ManipProcess()
{
	if(manipBannerSensor->GetVoltage()>3 && !BannerBroken && InOutSpeed < 0)
	{
		InOutSpeed = 0;
	}
	if(manipBannerSensor->GetVoltage()<3 && !BannerBroken && TwistSpeed != 0 && InOutSpeed == 0)
	{
		InOutSpeed = -1 * IN_OUT_SPEED_MOD;
	}
	
	if(InOutSpeed == 0 && TwistSpeed != 0)
	{
		InOutSpeed = .05;
	}
	
	//Combine inout with twist
	//Limit these?
	upperMotor->Set(-1*(InOutSpeed + TwistSpeed)/2);
	lowerMotor->Set((InOutSpeed - TwistSpeed)/2);
}

void ThunderManip::ManipTwist(float speed)
{
	TwistSpeed = speed * TWIST_SPEED_MOD;
}

void ThunderManip::ManipInOut(float selection)
{
	InOutSpeed = selection * IN_OUT_SPEED_MOD;	
}

void ThunderManip::ManipTwistAuto(PegPositions position)
{
	switch(position){
		case LOW:
		AutoGoalTime = GetTime() + LOW_PEG;
//		printf("SET TWIST TIME TO: %f   CUR TIME %f", AutoGoalTime, GetTime());
		break;
		case LOW_OFFSET:
		AutoGoalTime = GetTime() + LOW_PEG;
//		printf("SET TWIST TIME TO: %f   CUR TIME %f", AutoGoalTime, GetTime());
		break;
		case MEDIUM:
		AutoGoalTime = GetTime() + MEDIUM_PEG;
//		printf("SET TWIST TIME TO: %f   CUR TIME %f", AutoGoalTime, GetTime());
		break;
		case MEDIUM_OFFSET:
		AutoGoalTime = GetTime() + MEDIUM_PEG;
//		printf("SET TWIST TIME TO: %f   CUR TIME %f", AutoGoalTime, GetTime());
		break;
		case HIGH:
		AutoGoalTime = GetTime() + HIGH_PEG;
//		printf("SET TWIST TIME TO: %f   CUR TIME %f", AutoGoalTime, GetTime());
		break;
		case HIGH_OFFSET:
		AutoGoalTime = GetTime() + HIGH_PEG;
//		printf("SET TWIST TIME TO: %f   CUR TIME %f", AutoGoalTime, GetTime());
		break;
	}
}

void ThunderManip::ManipOutAuto()
{
	AutoOutTime = GetTime() + AUTO_OUT_TIME;
	//printf("SET OUT TIME TO: %f   CUR TIME %f", AutoOutTime, GetTime());
	/*tells currentManipAutoState to spit the tube out
	  set autoProgress to sensor
	  set autoGoal to 0
	  set autoStart to 1
	 */
}

void ThunderManip::ManipProcessAuto()
{
	if(GetTime() <= AutoGoalTime)
	{
		TwistSpeed = AUTO_TWIST_SPEED;
	}
	else
	{
		TwistSpeed = 0;
	}
	
	if(GetTime() <= AutoOutTime)
	{
		InOutSpeed = AUTO_OUT_SPEED;
	}
	else
	{
		InOutSpeed = 0;
	}
	/*checks currentAutoTask and determines what it needs to do from that
	  for ManipOutAuto, it tells motors to spin in the direction that spits the tube out
	  for ManipTwistAuto, it tells the motors to spin up if 0-x, spin down if 0 through -x
	  set autoProgress to number of rotations
	  out process is sensor value
	*/
	
	
	
}

bool ThunderManip::ManipCheckAutoDone() {
	if (GetTime() <= AutoGoalTime || GetTime() <= AutoOutTime){
		return false;
	}
	return true;
}

void ThunderManip::ManipResetAfterAuto() {
	TwistSpeed = 0;
	InOutSpeed = 0;
	AutoGoalTime = 0;
	AutoOutTime = 0;
}

float ThunderManip::ManipGetBannerSensor()
{
	return manipBannerSensor->GetVoltage();
}

bool ThunderManip::ManipLight()
{
	if(manipBannerSensor->GetVoltage()>3 && !BannerBroken)
	{
		return true;
	}
	return false;
}


void ThunderManip::ManipBannerSensorBroken(bool broken)
{
	BannerBroken = broken;
}

bool ThunderManip::ManipGetBannerSensorBroken()
{
	return BannerBroken;
}
