#ifndef THUNDER_MANIP_H
#define THUNDER_MANIP_H


//Auto times
//These #defines are used internally, there's no reason to use them outside of this class
#define LOW_PEG    				 1.5
#define MEDIUM_PEG  			 .75
#define HIGH_PEG   				 0

#define AUTO_OUT_TIME			 15	// let it run until the end of autonomous
#define AUTO_OUT_SPEED           1.0
#define AUTO_TWIST_SPEED		 -.75

#define IN_OUT_SPEED_MOD		 1.0
#define TWIST_SPEED_MOD			 1.0

#include "Resources.h"

/*
run the tube manipulator (wrists etc)
*/

class ThunderManip
{
public:
	ThunderManip();
	~ThunderManip();
	
	//Does all the calculations for the manipulator
	//THIS MUST BE CALLED CONSTANTLY IN AUTO AND TELE
	void ManipProcess();
	
	/*
	 * 
	 * THE NEXT FUNCTIONS ARE FOR TELEOPERATED ONLY AND SHOULD NOT BE USED IN AUTONOMOUS
	 * ... EVER.
	 * ... OF ALL TIME.
	 * 
	 */
	
	//Twists the manipulator up and down, according to the speed you give it
	//"speed" indicates direction and speed.
	//Valid values are between -1 and 1.
	//Negative values twist down when arm is in front of robot.
	//Positive values twist up when arm is in front of robot.
	//Not exclusive of ManipInOut.
	void ManipTwist(float speed);
	
	//Spins the manipulator to suck tubes in or out
	//"speed" determines speed and direction.
	//Valid values are between -1 and 1.
	//Negative values pull in.
	//Positive values push out.
	//Not exclusive of ManipTwist.
	void ManipInOut(float speed);
	
	/*
	 * 
	 * THE NEXT FUNCTIONS ARE FOR AUTONOMOUS ONLY AND SHOULD NOT BE USED IN TELEOP
	 * ... EVER.
	 * ... OF ALL TIME.
	 * 
	 */
	
	/*
	 * High level autonomous will call ManipProcessAuto constantly.
	 * 
	 * High level autonomous will call ManipTwistAuto or ManipOutAuto once.
	 * That will start the given operation. High level will wait until the operation is done.
	 * The operation is done when CheckAutoDone returns true. 
	 * 
	 * CheckAutoDone will continue to return true until the manipulator recieves a new task.
	 */
	
	typedef enum {LOW, LOW_OFFSET, MEDIUM, MEDIUM_OFFSET, HIGH, HIGH_OFFSET} PegPositions;
	
	//Twists the tube to prepare for the peg height you select
	//Goes until it's done
	void ManipTwistAuto(PegPositions position);
	
	//Spits tube out in auto
	//Goes until it's done
	void ManipOutAuto();
	
	
	//MUST BE CALLED CONSTANTLY IN AUTO
	void ManipProcessAuto();
	
	//Checks if most recent process is done
	bool ManipCheckAutoDone();
	
	//CALL THIS AFTER AUTONOMOUS IS DONE
	void ManipResetAfterAuto();
	
	//Returns the voltage of the banner sensor on the manip
	float ManipGetBannerSensor();
	
	//Returns true if we have a tube
	bool ManipLight();
	
	//Sets whether the got it sensor is broken
	void ManipBannerSensorBroken(bool broken);
	
	//Gets whether the got it sensor is broken
	bool ManipGetBannerSensorBroken();
	
	//typedef enum autoTask{manipOut, manipTwist, manipDoNothing};


private:
	float autoStart;
	float autoProgress;
	float autoGoal;
	float TwistSpeed;
	float InOutSpeed;
	double AutoGoalTime;
	double AutoOutTime;
	bool BannerBroken;
	AnalogChannel* manipBannerSensor;
	
	ThunderCANJaguar* upperMotor;
	ThunderCANJaguar* lowerMotor;
	
	
	//autoTask currentManipAutoState;
};


#endif
