#ifndef TRACKER_H
#define TRACKER_H

#include "WPILib.h"

// Threshold voltages -- provisional values
#define V_LOW 0.9
#define V_HIGH 4.0
// Number of states to average when getting buffered state
#define BUFFER_SIZE 5

class Tracker
{
	// private channels, since no one else will use these
	AnalogChannel sLeft, sCenter, sRight;
	int pastLeft[BUFFER_SIZE], pastCenter[BUFFER_SIZE],
		pastRight[BUFFER_SIZE];
	int totalStates;
	
public:
	//Possible states that tracking can be in
	typedef enum { WAY_OFF, ON_LINE, TOO_LEFT, TOO_RIGHT, WYE, STOP,
					SEMI_LEFT, SEMI_RIGHT } TrackState;
	
	//constructor
	Tracker::Tracker(int leftChannel, int centerChannel, int rightChannel);

	// show state of line tracking--are we on, off, to right or left?
	// (for autonomous)
	TrackState Tracker::GetState();
	// show state of line tracking -- but for each sensor take the
	// average of the last BUFFER_SIZE states; hopefully this will
	// reduce misreadings etc. ...
	TrackState Tracker::GetBufferedState();

	// Debug output of sensor values
	bool Tracker::GetLeft();
	bool Tracker::GetCenter();
	bool Tracker::GetRight();
	void Tracker::PrintState(TrackState state);
	
};

#endif
